@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: bbs_thread_ui.c,v 1.31 2003/11/22 19:17:16 fuyu Exp $
+ * $Id: bbs_thread_ui.c,v 1.33 2003/11/25 18:19:23 fuyu Exp $
  */
 
 #include "config.h"
@@ -142,6 +142,7 @@ typedef struct _RendererContext
   GHashTable *tag_table;
 
   GtkTextTag *thread_title_tag;
+  GtkTextTag *response_header_tag;
   GtkTextTag *response_name_tag;
   GtkTextTag *response_paragraph_tag;
   GtkTextTag *bold_tag;
@@ -209,6 +210,7 @@ PopupContext global_popup_context =
     NULL,				/* tag_table */
 
     NULL,				/* thread_title_tag */
+    NULL,				/* response_header_tag */
     NULL,				/* response_name_tag */
     NULL,				/* response_paragraph_tag */
     NULL,				/* bold_tag */
@@ -262,11 +264,183 @@ prepare_thread_ui_initialization(OchushaApplication *application)
 
 
 static GtkTextTag *thread_title_tag;
+static GtkTextTag *response_header_tag;
 static GtkTextTag *response_name_tag;
 static GtkTextTag *response_paragraph_tag;
 static GtkTextTag *bold_tag;
 
 
+static PangoFontDescription *thread_title_font_description = NULL;
+static PangoFontDescription *thread_body_font_description = NULL;
+static GtkWidget *font_selection_dialog = NULL;
+
+
+static void
+font_selection_dialog_response_cb(GtkWidget *dialog, gint response_id,
+				  OchushaApplication *application)
+{
+  GtkFontSelectionDialog *font_selection;
+  gchar *font_name = NULL;
+
+  g_return_if_fail(GTK_IS_FONT_SELECTION_DIALOG(dialog));
+  font_selection = GTK_FONT_SELECTION_DIALOG(dialog);
+
+  switch (response_id)
+    {
+    case GTK_RESPONSE_APPLY:
+      font_name = gtk_font_selection_dialog_get_font_name(font_selection);
+      preview_thread_view_font(font_name);
+      g_free(font_name);
+      return;
+
+    case GTK_RESPONSE_OK:
+      font_name	= gtk_font_selection_dialog_get_font_name(font_selection);
+      set_thread_view_font(font_name);
+      if (application->thread_view_font_name != NULL)
+	G_FREE(application->thread_view_font_name);
+      application->thread_view_font_name = font_name;
+      break;
+
+    case GTK_RESPONSE_CANCEL:
+      reset_thread_view_font(font_name);
+      break;
+    }
+
+  gtk_widget_destroy(dialog);
+}
+
+
+void
+select_thread_view_font(OchushaApplication *application)
+{
+  char *font_name;
+  if (font_selection_dialog != NULL)
+    return;
+
+  font_selection_dialog
+    = gtk_font_selection_dialog_new(_("Thread View Font Selection"));
+
+  font_name = application->thread_view_font_name;
+
+  if (font_name != NULL && *font_name != '\0')
+    gtk_font_selection_dialog_set_font_name(GTK_FONT_SELECTION_DIALOG(font_selection_dialog),
+					    font_name);
+
+  g_signal_connect(G_OBJECT(font_selection_dialog), "response",
+		   G_CALLBACK(font_selection_dialog_response_cb), application);
+  g_signal_connect(G_OBJECT(font_selection_dialog), "destroy",
+		   G_CALLBACK(gtk_widget_destroyed), &font_selection_dialog);
+
+  gtk_widget_show(GTK_FONT_SELECTION_DIALOG(font_selection_dialog)->apply_button);
+  gtk_widget_show_all(font_selection_dialog);
+}
+
+
+void
+set_thread_view_font(const char *font_name)
+{
+  g_return_if_fail(thread_title_tag != NULL);
+  g_return_if_fail(response_header_tag != NULL);
+  g_return_if_fail(response_paragraph_tag != NULL);
+
+  if (font_name == NULL || *font_name == '\0')
+    return;
+
+  if (thread_title_font_description != NULL)
+    {
+      pango_font_description_free(thread_title_font_description);
+      thread_title_font_description = NULL;
+    }
+  if (thread_body_font_description != NULL)
+    {
+      pango_font_description_free(thread_body_font_description);
+      thread_body_font_description = NULL;
+    }
+
+  thread_body_font_description = pango_font_description_from_string(font_name);
+  g_object_set(G_OBJECT(response_paragraph_tag),
+	       "font-desc", thread_body_font_description,
+	       NULL);
+  g_object_set(G_OBJECT(response_header_tag),
+	       "font-desc", thread_body_font_description,
+	       NULL);
+  thread_title_font_description
+    = pango_font_description_copy(thread_body_font_description);
+  pango_font_description_set_size(thread_title_font_description,
+				  pango_font_description_get_size(thread_body_font_description) * 1.2);
+  g_object_set(G_OBJECT(thread_title_tag),
+	       "font-desc", thread_title_font_description,
+	       NULL);
+}
+
+
+void reset_thread_view_font()
+{
+  g_return_if_fail(thread_title_tag != NULL);
+  g_return_if_fail(response_header_tag != NULL);
+  g_return_if_fail(response_paragraph_tag != NULL);
+
+  if (thread_title_font_description == NULL
+      || thread_body_font_description == NULL)
+    {
+      PangoFontDescription *default_desc
+	= pango_font_description_new();
+
+      g_object_set(G_OBJECT(response_paragraph_tag),
+		   "font-desc", default_desc,
+		   NULL);
+      g_object_set(G_OBJECT(response_header_tag),
+		   "font-desc", default_desc,
+		   NULL);
+      g_object_set(G_OBJECT(thread_title_tag),
+		   "font-desc", default_desc,
+		   NULL);
+      pango_font_description_free(default_desc);
+
+      return;
+    }
+
+  g_object_set(G_OBJECT(response_paragraph_tag),
+	       "font-desc", thread_body_font_description,
+	       NULL);
+  g_object_set(G_OBJECT(response_header_tag),
+	       "font-desc", thread_body_font_description,
+	       NULL);
+  g_object_set(G_OBJECT(thread_title_tag),
+	       "font-desc", thread_title_font_description,
+	       NULL);
+}
+
+
+void
+preview_thread_view_font(const char *font_name)
+{
+  PangoFontDescription *font_desc;
+  g_return_if_fail(thread_title_tag != NULL);
+  g_return_if_fail(response_header_tag != NULL);
+  g_return_if_fail(response_paragraph_tag != NULL);
+
+  if (font_name == NULL || *font_name == '\0')
+    return;
+
+  font_desc = pango_font_description_from_string(font_name);
+  g_object_set(G_OBJECT(response_paragraph_tag),
+	       "font-desc", font_desc,
+	       NULL);
+  g_object_set(G_OBJECT(response_header_tag),
+	       "font-desc", font_desc,
+	       NULL);
+
+  pango_font_description_set_size(font_desc,
+			pango_font_description_get_size(font_desc) * 1.2);
+  pango_font_description_set_weight(font_desc, PANGO_WEIGHT_BOLD);
+  g_object_set(G_OBJECT(thread_title_tag),
+	       "font-desc", font_desc,
+	       NULL);
+  pango_font_description_free(font_desc);
+}
+
+
 static void
 initialize_popup_context(PopupContext *popup_context,
 			 OchushaApplication *application)
@@ -291,6 +465,7 @@ initialize_popup_context(PopupContext *popup_context,
   popup_context->renderer_context.application = application;
 
   popup_context->renderer_context.thread_title_tag = thread_title_tag;
+  popup_context->renderer_context.response_header_tag = response_header_tag;
   popup_context->renderer_context.response_name_tag = response_name_tag;
   popup_context->renderer_context.response_paragraph_tag
     = response_paragraph_tag;
@@ -383,6 +558,8 @@ initialize_thread_ui(OchushaApplication *application)
 
   thread_title_tag
     = bbs_thread_view_get_tag_by_name(dummy_view, "thread_title");
+  response_header_tag
+    = bbs_thread_view_get_tag_by_name(dummy_view, "response_header");
   response_name_tag
     = bbs_thread_view_get_tag_by_name(dummy_view, "response_name");
   response_paragraph_tag
@@ -403,6 +580,10 @@ initialize_thread_ui(OchushaApplication *application)
     g_hash_table_insert(tag_table, "b", bold_tag);
   }
 
+  if (application->thread_view_font_name != NULL
+      && *application->thread_view_font_name != '\0')
+    set_thread_view_font(application->thread_view_font_name);
+
 #if !ENABLE_MULTIPLE_POPUPS
   initialize_popup_context(&global_popup_context, application);
 #endif
@@ -869,15 +1050,16 @@ render_bbs_thread(WorkerThread *employee, gpointer args)
       tag_table,				/* tag_table */
 
       bbs_thread_view_get_tag_by_name(job_args->view, "thread_title"),
+      bbs_thread_view_get_tag_by_name(job_args->view, "response_header"),
       bbs_thread_view_get_tag_by_name(job_args->view, "response_name"),
       bbs_thread_view_get_tag_by_name(job_args->view, "response_paragraph"),
       bbs_thread_view_get_tag_by_name(job_args->view, "bold"),
   };
 
 #if defined(ENABLE_REGEXP) && defined(HAVE_ONIGURUMA)
-  /* "\357\274\214\343\200\201"  "" UTF-8ɽ */
+  /* "\357\274\214\343\200\201\357\274\235"  "" UTF-8ɽ */
   if (regcomp(&context.comma_separated_link_pattern,
-	      "^[,\357\274\214\343\200\201][ ]*([1-9][0-9]*)(-[1-9][0-9]*)?",
+	      "^[,=\357\274\214\343\200\201\357\274\235][ ]*([1-9][0-9]*)(-[1-9][0-9]*)?",
 	      REG_EXTENDED))
     {
       fprintf(stderr, "invalid regular expression\n");
@@ -1075,6 +1257,7 @@ render_response(OchushaBBSThread *thread, gint number,
       gdk_threads_enter();
     }
 #endif
+  bbs_thread_view_push_tag(view, context->response_header_tag);
   bbs_thread_view_append_text(view, buffer, -1);
   bbs_thread_view_append_text(view, LARGE_COLON, 3);
 
@@ -1085,6 +1268,8 @@ render_response(OchushaBBSThread *thread, gint number,
       parse_text(&context->handler, context, response->name, -1);
       bbs_thread_view_pop_tags(view, context->bold_tag);
       bbs_thread_view_pop_tags(view, context->response_name_tag);
+      if (context->show_mailto_literally)
+	bbs_thread_view_append_text(view, " []", 3);
     }
   else
     {
@@ -1123,7 +1308,7 @@ render_response(OchushaBBSThread *thread, gint number,
   parse_text(&context->handler, context, response->date_id, -1);
 
   bbs_thread_view_append_text(view, "\n", 1);
-
+  bbs_thread_view_pop_tags(view, context->response_header_tag);
 
   bbs_thread_view_push_tag(view, context->response_paragraph_tag);
   cur_pos = response->content;
