@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: bbs_thread_view.c,v 1.18 2003/11/21 21:03:31 fuyu Exp $
+ * $Id: bbs_thread_view.c,v 1.19 2003/11/25 18:19:23 fuyu Exp $
  */
 
 #include "config.h"
@@ -212,14 +212,19 @@ bbs_thread_view_class_init(BBSThreadViewClass *klass)
   tag = gtk_text_tag_new("thread_title");
   g_object_set(G_OBJECT(tag),
 	       "foreground", "red",
+#if 0
 	       "weight", PANGO_WEIGHT_BOLD,
 	       "scale", PANGO_SCALE_X_LARGE,
+#endif
 	       NULL);
   gtk_text_tag_table_add(default_tag_table, tag);
 
+  tag = gtk_text_tag_new("response_header");
+  gtk_text_tag_table_add(default_tag_table, tag);
+
   tag = gtk_text_tag_new("response_name");
   g_object_set(G_OBJECT(tag),
-	       "foreground", "green",
+	       "foreground", "forestgreen",
 	       NULL);
   gtk_text_tag_table_add(default_tag_table, tag);
 
@@ -283,12 +288,15 @@ bbs_thread_view_class_init(BBSThreadViewClass *klass)
 	       NULL);
   gtk_text_tag_table_add(default_tag_table, tag);
 
+#if 0
   tag = gtk_text_tag_new("match-string");
   g_object_set(G_OBJECT(tag),
 	       "foreground", "red",
 	       "background", "#ffffd0",
 	       "weight", PANGO_WEIGHT_BOLD,
 	       NULL);
+  gtk_text_tag_table_add(default_tag_table, tag);
+#endif
 			 
   bbs_thread_view_signals[LINK_MOUSE_OVER_SIGNAL] =
     g_signal_new("link_mouse_over",
