@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: board_properties.c,v 1.5 2003/11/22 19:17:16 fuyu Exp $
+ * $Id: board_properties.c,v 1.6 2003/11/25 22:37:39 fuyu Exp $
  */
 
 #include "config.h"
@@ -149,15 +149,15 @@ open_board_properties(OchushaApplication *application,
 				       application->top_level,
 				       GTK_DIALOG_DESTROY_WITH_PARENT,
 
+				       GTK_STOCK_CANCEL,
+				       GTK_RESPONSE_CANCEL,
+
 				       GTK_STOCK_APPLY,
 				       GTK_RESPONSE_APPLY,
 
 				       GTK_STOCK_OK,
 				       GTK_RESPONSE_OK,
 
-				       GTK_STOCK_CANCEL,
-				       GTK_RESPONSE_CANCEL,
-
 				       NULL);
   gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_OK);
   g_signal_connect(G_OBJECT(dialog), "response",
@@ -448,7 +448,7 @@ static GtkWidget *
 build_filter_setting_frame(OchushaApplication *application,
 			   OchushaBulletinBoard *board, DialogStates *states)
 {
-  GtkWidget *filter_frame = gtk_frame_new(_("Filter Setting"));
+  GtkWidget *filter_frame = gtk_frame_new(_("Threadlist's Filter Setting"));
 
   GtkWidget *vbox;
   GtkWidget *tmp_widget;
