@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: boardlist_ui.c,v 1.26 2003/11/23 22:51:13 fuyu Exp $
+ * $Id: boardlist_ui.c,v 1.30 2003/11/26 04:13:26 fuyu Exp $
  */
 
 #include "config.h"
@@ -458,6 +458,9 @@ make_new_thread(OchushaApplication *application)
   g_return_if_fail(application != NULL
 		   && application->contents_window != NULL);
 
+  if (application->config.offline)
+    return;
+
   contents_window = PANED_NOTEBOOK(application->contents_window);
 
   board = (OchushaBulletinBoard *)paned_notebook_get_current_item(contents_window);
@@ -1085,6 +1088,10 @@ page_switched_cb(PanedNotebook *paned_notebook,
 
       g_return_if_fail(board != NULL);
       info = ensure_bulletin_board_info(board, application);
+      paned_notebook_set_tooltips(PANED_NOTEBOOK(new_page),
+				  application->thread_tab_enable_tooltips);
+      paned_notebook_set_paned_style(PANED_NOTEBOOK(new_page),
+				     application->threadlist_pane_style);
       gtk_paned_set_position(GTK_PANED(new_page),
 			     application->threadlist_height);
       gtk_toggle_button_set_active(application->filter_button,
@@ -1169,11 +1176,17 @@ boardlist_write_category_element_cb(OchushaBBSTable *table,
 	  "        </attribute>\n", (board)->attribute)
 
 #define OUTPUT_BOARD_ATTRIBUTE_STRING(file, board, attribute)		\
-  if ((board)->attribute != NULL)					\
-    fprintf(file,							\
-	  "        <attribute name=\"" #attribute	"\">\n"		\
-	  "          <string>%s</string>\n"				\
-	  "        </attribute>\n", (board)->attribute)
+  do {									\
+    if ((board)->attribute != NULL)					\
+      {									\
+	gchar *text = g_markup_escape_text((board)->attribute, -1);	\
+	fprintf(file,							\
+		"        <attribute name=\"" #attribute	"\">\n"		\
+		"          <string>%s</string>\n"			\
+		"        </attribute>\n", text);			\
+	g_free(text);							\
+      }									\
+  } while (0)
 
 
 static void
@@ -1450,7 +1463,7 @@ new_thread_dialog_response_cb(GtkWidget *dialog, gint response_id,
 	  && message != NULL && *message != '\0')
 	{
 	  result = ochusha_bulletin_board_create_new_thread(board,
-						&application->config,
+						application->broker,
 						title, &response);
 	  g_free(message);
 	  if (result)
