@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: boardlist_view.c,v 1.10 2003/11/07 14:27:06 fuyu Exp $
+ * $Id: boardlist_view.c,v 1.11 2003/11/23 17:07:26 fuyu Exp $
  */
 
 #include "config.h"
@@ -114,7 +114,7 @@ typedef struct _TreeNodeInfo
 
 
 enum {
-  OPEN_VIEW_SIGNAL,
+  OPEN_THREADLIST_VIEW_SIGNAL,
 
   SELECT_CATEGORY_SIGNAL,
   CATEGORY_COLLAPSED_SIGNAL,
@@ -146,6 +146,7 @@ enum {
 enum {
   POPUP_MENU_OPEN_SELECTION = 1,
   POPUP_MENU_OPEN_SELECTION_IN_TAB,
+  POPUP_MENU_OPEN_SELECTION_WITH_BROWSER,
   POPUP_MENU_TOGGLE_HIDE_SELECTION,
   POPUP_MENU_KILL_SELECTION
 };
@@ -203,12 +204,19 @@ boardlist_view_class_init(BoardlistViewClass *klass)
 	NULL					/* type */
       },
       {
-	_("/Open Selection In _Tab"),		/* menu path */
+	_("/Open Selected Boards in _Tab"),	/* menu path */
 	NULL,					/* accelerator */
 	popup_menu_callback,			/* callback_func */
 	POPUP_MENU_OPEN_SELECTION_IN_TAB,	/* act */
 	NULL					/* type */
       },
+      {
+	_("/Open Selected Boards with Web Browser"),	/* menu path */
+	NULL,					/* accelerator */
+	popup_menu_callback,			/* callback_func */
+	POPUP_MENU_OPEN_SELECTION_WITH_BROWSER,	/* act */
+	NULL					/* type */
+      },
       {
 	_("/-------"),
 	NULL,
@@ -248,15 +256,16 @@ boardlist_view_class_init(BoardlistViewClass *klass)
   /* GtkObject signals */
   object_class->destroy = boardlist_view_destroy;
 
-  boardlist_view_signals[OPEN_VIEW_SIGNAL] =
-    g_signal_new("open_view",
+  boardlist_view_signals[OPEN_THREADLIST_VIEW_SIGNAL] =
+    g_signal_new("open_threadlist_view",
 		 G_TYPE_FROM_CLASS(klass),
 		 G_SIGNAL_RUN_LAST,
-		 G_STRUCT_OFFSET(BoardlistViewClass, open_view),
+		 G_STRUCT_OFFSET(BoardlistViewClass, open_threadlist_view),
 		 NULL, NULL,
-		 ochusha_marshal_VOID__POINTER_BOOLEAN,
-		 G_TYPE_NONE, 2,
-		 G_TYPE_POINTER,
+		 ochusha_marshal_VOID__OBJECT_BOOLEAN_BOOLEAN,
+		 G_TYPE_NONE, 3,
+		 OCHUSHA_TYPE_BULLETIN_BOARD,
+		 G_TYPE_BOOLEAN,
 		 G_TYPE_BOOLEAN);
 
   boardlist_view_signals[SELECT_CATEGORY_SIGNAL] =
@@ -411,7 +420,7 @@ boardlist_view_class_init(BoardlistViewClass *klass)
 			       G_TYPE_INT, -1);
 			       
 
-  klass->open_view = NULL;
+  klass->open_threadlist_view = NULL;
 
   klass->select_category = NULL;
   klass->category_expanded = NULL;
@@ -604,10 +613,21 @@ do_on_selection_helper(GtkTreeModel *model, GtkTreePath *path,
 	gtk_tree_view_expand_row(&args->view->view, path, TRUE);
       else
 	g_signal_emit(G_OBJECT(args->view),
-		      boardlist_view_signals[OPEN_VIEW_SIGNAL],
+		      boardlist_view_signals[OPEN_THREADLIST_VIEW_SIGNAL],
 		      0,
 		      data,
-		      what_to_do == POPUP_MENU_OPEN_SELECTION_IN_TAB);
+		      what_to_do == POPUP_MENU_OPEN_SELECTION_IN_TAB,
+		      FALSE);
+      break;
+
+    case POPUP_MENU_OPEN_SELECTION_WITH_BROWSER:
+      if (OCHUSHA_IS_BULLETIN_BOARD(data))
+	g_signal_emit(G_OBJECT(args->view),
+		      boardlist_view_signals[OPEN_THREADLIST_VIEW_SIGNAL],
+		      0,
+		      data,
+		      FALSE,
+		      TRUE);
       break;
 
     case POPUP_MENU_TOGGLE_HIDE_SELECTION:
@@ -832,10 +852,11 @@ tree_view_button_press_event_cb(GtkWidget *widget, GdkEventButton *event,
 
     case BOARDLIST_BOARD_ROW:
       g_signal_emit(G_OBJECT(view),
-		    boardlist_view_signals[OPEN_VIEW_SIGNAL],
+		    boardlist_view_signals[OPEN_THREADLIST_VIEW_SIGNAL],
 		    0,
 		    data,
-		    event->button == 2);
+		    event->button == 2,
+		    FALSE);
       break;
 
     default:
