@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: bulletin_board_ui.c,v 1.49 2003/11/23 22:39:35 fuyu Exp $
+ * $Id: bulletin_board_ui.c,v 1.52 2003/11/26 04:13:26 fuyu Exp $
  */
 
 #include "config.h"
@@ -104,6 +104,8 @@ static void get_current_thread(OchushaApplication *application,
 			       IconLabel **tab_label);
 static void write_response_button_cb(GtkWidget *widget,
 				     OchushaApplication *application);
+static void select_font_button_cb(GtkWidget *widget,
+				  OchushaApplication *application);
 static void refresh_thread_button_cb(GtkWidget *widget,
 				     OchushaApplication *application);
 static void start_thread_search_button_cb(GtkWidget *widget,
@@ -248,11 +250,18 @@ threadlist_read_thread_element_cb(OchushaBulletinBoard *board,
 	  "      </attribute>\n", (thread)->attribute)
 
 #define OUTPUT_THREAD_ATTRIBUTE_STRING(file, thread, attribute)		\
-  if ((thread)->attribute != NULL)					\
-    fprintf(file,							\
-	  "      <attribute name=\"" #attribute	"\">\n"			\
-	  "        <string>%s</string>\n"				\
-	  "      </attribute>\n", (thread)->attribute)
+  do {									\
+    if ((thread)->attribute != NULL)					\
+      {									\
+	gchar *text = g_markup_escape_text((thread)->attribute, -1);	\
+	fprintf(file,							\
+		"      <attribute name=\"" #attribute	"\">\n"		\
+		"        <string>%s</string>\n"				\
+		"      </attribute>\n", text);				\
+	g_free(text);							\
+      }									\
+  } while (0)
+
 
 
 static void
@@ -580,6 +589,14 @@ open_bulletin_board(OchushaApplication *application,
 			       application,
 			       0);
 
+      gtk_toolbar_prepend_space(toolbar);
+      gtk_toolbar_insert_stock(toolbar, GTK_STOCK_SELECT_FONT,
+			       _("Select font to render threads"),
+			       "select_font_to_render_threads",
+			       GTK_SIGNAL_FUNC(select_font_button_cb),
+			       application,
+			       0);
+
       gtk_toolbar_prepend_space(toolbar);
       gtk_toolbar_insert_stock(toolbar, GTK_STOCK_REFRESH,
 			       _("Refresh current thread"),
@@ -1684,7 +1701,7 @@ write_dialog_response_cb(GtkWidget *write_dialog, gint response_id,
       if (message != NULL && *message != '\0')
 	{
 	  result = ochusha_bbs_thread_post_response(thread,
-						    &application->config,
+						    application->broker,
 						    &response);
 	  g_free(message);
 	  if (result)
@@ -1793,10 +1810,16 @@ write_response(OchushaApplication *application)
   IconLabel *tab_label;
   GtkWidget *write_dialog;
 
+  if (application->config.offline)
+    return;
+
   get_current_thread(application, &view, &thread, &tab_label);
   if (view == NULL || thread == NULL)
     return;
 
+  if (thread->flags & OCHUSHA_BBS_THREAD_DAT_DROPPED)
+    return;
+
   if (!ochusha_bbs_thread_is_post_supported(thread))
     {
       ochusha_open_url(application,
@@ -1845,6 +1868,13 @@ write_response_button_cb(GtkWidget *widget, OchushaApplication *application)
 }
 
 
+static void
+select_font_button_cb(GtkWidget *widget, OchushaApplication *application)
+{
+  select_thread_view_font(application);
+}
+
+
 static void
 refresh_thread_button_cb(GtkWidget *widget, OchushaApplication *application)
 {
