@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_ui.c,v 1.27 2003/11/23 22:04:09 fuyu Exp $
+ * $Id: ochusha_ui.c,v 1.32 2003/11/26 06:03:08 fuyu Exp $
  */
 
 #include "config.h"
@@ -115,7 +115,10 @@ static gboolean access_failed_cb(OchushaNetworkBroker *broker,
 				 OchushaNetworkBrokerFailureReason reason_code,
 				 const gchar *reason,
 				 OchushaApplication *application);
+static void output_log_cb(OchushaNetworkBroker *broker, const gchar *message,
+			  OchushaApplication *application);
 
+static void open_preference_dialog(OchushaApplication *application);
 static void open_about_dialog(OchushaApplication *application);
 static void about_dialog_response_cb(GtkWidget *dialog, gint response_id,
 				     gpointer unused);
@@ -149,6 +152,7 @@ static pthread_mutex_t main_ui_lock;
       abort();						\
     }
 
+static GtkWidget *preference_dialog = NULL;
 static GtkWidget *about_dialog = NULL;
 static GdkPixbuf *banner_pixbuf = NULL;
 static GdkPixbuf *banner1_pixbuf = NULL;
@@ -205,7 +209,7 @@ ochusha_init_application(OchushaApplication *application)
       NULL
     },
     {
-      _("/Board/Hide hidden boardlist items"),
+      _("/Board/Hide Hidden Boardlist Items"),
       NULL,
       main_menu_callback,
       MAIN_MENU_HIDE_HIDDEN_BOARDLIST_ITEMS,
@@ -254,12 +258,42 @@ ochusha_init_application(OchushaApplication *application)
       NULL
     },
     {
-      _("/Thread/Hide hidden threads"),
+      _("/Thread/Hide Hidden Threads"),
       NULL,
       main_menu_callback,
       MAIN_MENU_HIDE_HIDDEN_THREADS,
       "<CheckItem>",
     },
+    {
+      _("/Tools"),
+      NULL,
+      NULL,
+      0,
+      "<Branch>"
+    },
+#if 0
+    {
+      _("/Tools/Debug Console"),
+      NULL,
+      main_menu_callback,
+      MAIN_MENU_DEBUG_CONSOLE,
+      NULL,
+    },
+    {
+      _("/Tools/-------"),
+      NULL,
+      NULL,
+      0,
+      "<Separator>"
+    },
+#endif
+    {
+      _("/Tools/Options"),
+      NULL,
+      main_menu_callback,
+      MAIN_MENU_OPTIONS,
+      NULL,
+    },
     {
       _("/_Help"),
       NULL,
@@ -343,9 +377,11 @@ ochusha_init_application(OchushaApplication *application)
   application->menu_bar = gtk_item_factory_get_widget(main_menu_item_factory,
 						      "<main>");
   /* ̤ʵǽΥ˥塼򻦤 */
-  /* ochusha_set_main_menu_sensitive(MAIN_MENU_OPEN_URL, FALSE); */
   ochusha_set_main_menu_sensitive(MAIN_MENU_REFRESH_THREAD_ALL, FALSE);
-  /* ochusha_set_main_menu_sensitive(MAIN_MENU_ABOUT, FALSE); */
+#if 0
+  ochusha_set_main_menu_sensitive(MAIN_MENU_DEBUG_CONSOLE, FALSE);
+  ochusha_set_main_menu_sensitive(MAIN_MENU_OPTIONS, FALSE);
+#endif
 
   item = GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget_by_action(
 				main_menu_item_factory,
@@ -786,6 +822,15 @@ main_menu_callback(gpointer data, guint action, GtkWidget *widget)
       refresh_current_threadlist_all(application);
       return;
 
+#if 0
+    case MAIN_MENU_DEBUG_CONSOLE:
+      return;
+#endif
+
+    case MAIN_MENU_OPTIONS:
+      open_preference_dialog(application);
+      return;
+
     case MAIN_MENU_ABOUT:
       open_about_dialog(application);
       return;
@@ -807,12 +852,18 @@ main_menu_callback(gpointer data, guint action, GtkWidget *widget)
 	  "      <boolean val=\"%s\"/>\n"				\
 	  "    </attribute>\n", (config)->attribute ? "true" : "false" )
 
-#define OUTPUT_CONFIG_ATTRIBUTE_STRING(file, config, attribute)	\
-  if ((config)->attribute != NULL)				\
-    fprintf(file,						\
-	  "    <attribute name=\"" #attribute	"\">\n"		\
-	  "      <string>%s</string>\n"				\
-	  "    </attribute>\n", (config)->attribute)
+#define OUTPUT_CONFIG_ATTRIBUTE_STRING(file, config, attribute)		\
+  do {									\
+    if ((config)->attribute != NULL)					\
+      {									\
+	gchar *text = g_markup_escape_text((config)->attribute, -1);	\
+	fprintf(file,							\
+		"    <attribute name=\"" #attribute	"\">\n"		\
+		"      <string>%s</string>\n"				\
+		"    </attribute>\n", text);				\
+	g_free(text);							\
+      }									\
+  } while (0)
 
 
 static void
@@ -847,6 +898,7 @@ write_optional_prefs(FILE *file, gpointer user_data)
 				  show_threadlist_entry_when_thread_selected);
   OUTPUT_CONFIG_ATTRIBUTE_STRING(file, application, threadlist_view_contents);
 
+  OUTPUT_CONFIG_ATTRIBUTE_STRING(file, application, thread_view_font_name);
   OUTPUT_CONFIG_ATTRIBUTE_INT(file, application, show_mailto_mode);
 
   OUTPUT_CONFIG_ATTRIBUTE_BOOLEAN(file, application, enable_popup_title);
@@ -895,6 +947,8 @@ configure_application(OchushaApplication *application)
 		   G_CALLBACK(access_terminated_cb), application);
   g_signal_connect(G_OBJECT(application->broker), "access_failed",
 		   G_CALLBACK(access_failed_cb), application);
+  g_signal_connect(G_OBJECT(application->broker), "output_log",
+		   G_CALLBACK(output_log_cb), application);
   /* Sets unspecified attributes */
   if (application->width == 0)
     {	/* ƵưȻפ */
@@ -932,6 +986,7 @@ configure_application(OchushaApplication *application)
 
 
       /* ɽϢ */
+      application->thread_view_font_name = NULL;
       application->show_mailto_mode = OCHUSHA_DEFAULT_SHOW_MAILTO_MODE;
 
 
@@ -989,7 +1044,7 @@ configure_application(OchushaApplication *application)
 
   if (application->web_browser_template == NULL)
     application->web_browser_template
-      = g_strdup(OCHUSHA_DEFAULT_WEB_BROWSER_TEMPLATE);
+      = G_STRDUP(OCHUSHA_DEFAULT_WEB_BROWSER_TEMPLATE);
 }
 
 
@@ -1108,6 +1163,9 @@ read_optional_prefs(GHashTable *pref_attrs, gpointer user_data)
     application->threadlist_view_contents
       = G_STRDUP(OCHUSHA_DEFAULT_THREADLIST_VIEW_CONTENTS);
 
+  application->thread_view_font_name
+    = ochusha_utils_get_attribute_string(pref_attrs, "thread_view_font_name");
+
   application->show_mailto_mode
     = (ThreadViewMailtoMode)ochusha_utils_get_attribute_int(pref_attrs,
 							   "show_mailto_mode");
@@ -1323,10 +1381,598 @@ access_failed_cb(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer,
 }
 
 
+static void
+output_log_cb(OchushaNetworkBroker *broker, const gchar *message,
+	      OchushaApplication *application)
+{
+#if 0
+  fprintf(stderr, "LOG: %s", message);
+#endif
+}
+
+
+static void
+preference_dialog_response_cb(GtkWidget *widget, gint response_id,
+			      OchushaApplication *application)
+{
+  switch (response_id)
+    {
+    case GTK_RESPONSE_APPLY:
+    case GTK_RESPONSE_OK:
+      {
+	PanedNotebook *boardlist_pane
+	  = PANED_NOTEBOOK(application->contents_window);
+	GtkWidget *current_board_pane
+	  = paned_notebook_get_current_item_view(boardlist_pane);
+	gpointer tmp_data;
+
+	tmp_data = g_object_get_data(G_OBJECT(widget), "web_browser_template");
+	if (tmp_data != NULL)
+	  {
+	    const gchar *template = gtk_entry_get_text(GTK_ENTRY(tmp_data));
+	    if (template != NULL && *template != '\0')
+	      {
+		G_FREE(application->web_browser_template);
+		application->web_browser_template = G_STRDUP(template);
+	      }
+	  }
+
+	tmp_data = g_object_get_data(G_OBJECT(widget),
+				     "board_tab_enable_tooltips");
+	if (tmp_data != NULL)
+	  {
+	    application->board_tab_enable_tooltips
+	      = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(tmp_data));
+	    paned_notebook_set_tooltips(boardlist_pane,
+				application->board_tab_enable_tooltips);
+	  }
+
+#if 0	/* ɤ */
+	tmp_data = g_object_get_data(G_OBJECT(widget),
+				     "boardlist_pane_style");
+	if (tmp_data != NULL)
+	  {
+	    application->boardlist_pane_style
+	      = (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(tmp_data))
+		 ? VPANED_STYLE : HPANED_STYLE);
+	    paned_notebook_set_paned_style(boardlist_pane,
+					   application->boardlist_pane_style);
+	  }
+#endif
+
+	tmp_data = g_object_get_data(G_OBJECT(widget),
+				     "threadlist_pane_style");
+	if (tmp_data != NULL)
+	  {
+	    application->threadlist_pane_style
+	      = (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(tmp_data))
+		 ? VPANED_STYLE : HPANED_STYLE);
+	    if (current_board_pane != NULL)
+	      paned_notebook_set_paned_style(PANED_NOTEBOOK(current_board_pane), application->threadlist_pane_style);
+	  }
+
+	tmp_data = g_object_get_data(G_OBJECT(widget), "enable_proxy");
+	if (tmp_data != NULL)
+	  {
+	    application->config.enable_proxy
+	      = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(tmp_data));
+	  }
+
+	tmp_data = g_object_get_data(G_OBJECT(widget),
+				     "enable_proxy_only_for_posting");
+	if (tmp_data != NULL)
+	  {
+	    application->config.enable_proxy_only_for_posting
+	      = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(tmp_data));
+	  }
+
+	tmp_data = g_object_get_data(G_OBJECT(widget), "proxy_url");
+	if (tmp_data != NULL)
+	  {
+	    const gchar *url;
+	    if (application->config.proxy_url != NULL)
+	      G_FREE(application->config.proxy_url);
+	    url = gtk_entry_get_text(GTK_ENTRY(tmp_data));
+	    if (*url == '\0')
+	      application->config.proxy_url = NULL;
+	    else
+	      application->config.proxy_url = G_STRDUP(url);
+	  }
+
+	tmp_data = g_object_get_data(G_OBJECT(widget), "proxy_user");
+	if (tmp_data != NULL)
+	  {
+	    const gchar *name;
+	    if (application->config.proxy_user != NULL)
+	      G_FREE(application->config.proxy_user);
+	    name = gtk_entry_get_text(GTK_ENTRY(tmp_data));
+	    if (*name == '\0')
+	      application->config.proxy_user = NULL;
+	    else
+	      application->config.proxy_user = G_STRDUP(name);
+	  }
+
+	tmp_data = g_object_get_data(G_OBJECT(widget), "proxy_password");
+	if (tmp_data != NULL)
+	  {
+	    const gchar *password;
+	    if (application->config.proxy_password != NULL)
+	      G_FREE(application->config.proxy_password);
+	    password = gtk_entry_get_text(GTK_ENTRY(tmp_data));
+	    if (*password == '\0')
+	      application->config.proxy_password = NULL;
+	    else
+	      application->config.proxy_password = G_STRDUP(password);
+	  }
+
+	tmp_data = g_object_get_data(G_OBJECT(widget), "enable_popup_title");
+	if (tmp_data != NULL)
+	  {
+	    application->enable_popup_title
+	      = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(tmp_data));
+	  }
+
+	tmp_data = g_object_get_data(G_OBJECT(widget),
+				     "thread_tab_enable_tooltips");
+	if (tmp_data != NULL)
+	  {
+	    application->thread_tab_enable_tooltips
+	      = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(tmp_data));
+	    if (current_board_pane != NULL)
+	      paned_notebook_set_tooltips(PANED_NOTEBOOK(current_board_pane),
+				application->thread_tab_enable_tooltips);
+	  }
+
+	tmp_data = g_object_get_data(G_OBJECT(widget),
+				     "show_threadlist_entry");
+	if (tmp_data != NULL)
+	  {
+	    application->show_threadlist_entry_when_thread_selected
+	      = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(tmp_data));
+	  }
+
+	tmp_data = g_object_get_data(G_OBJECT(widget), "filter_setting");
+
+	if (tmp_data != NULL)
+	  {
+	    const ThreadlistFilter *filter;
+	    filter = threadlist_filter_setting_get_filter(THREADLIST_FILTER_SETTING(tmp_data));
+	    G_FREE(application->user_default_filter.rule);
+	    application->user_default_filter.rule = G_STRDUP(filter->rule);
+	    application->user_default_filter.ignore_threshold
+	      = filter->ignore_threshold;
+	  }
+
+	tmp_data = g_object_get_data(G_OBJECT(widget),
+				     "enable_popup_response");
+	if (tmp_data != NULL)
+	  {
+	    application->enable_popup_response
+	      = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(tmp_data));
+	  }
+
+	tmp_data = g_object_get_data(G_OBJECT(widget),
+				     "popup_response_delay");
+	if (tmp_data != NULL)
+	  {
+	    application->popup_response_delay
+	      = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(tmp_data));
+	  }
+
+	tmp_data = g_object_get_data(G_OBJECT(widget),
+				     "popup_close_delay");
+	if (tmp_data != NULL)
+	  {
+	    application->popup_close_delay
+	      = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(tmp_data));
+	  }
+
+	tmp_data = g_object_get_data(G_OBJECT(widget),
+				     "show_mailto_literally");
+	if (tmp_data != NULL)
+	  {
+	    application->show_mailto_mode
+	      = (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(tmp_data))
+		 ? THREAD_VIEW_MAILTO_MODE_SHOW
+		 : THREAD_VIEW_MAILTO_MODE_HIDE);
+	  }
+
+	break;
+      }
+
+    case GTK_RESPONSE_CANCEL:
+      break;
+    }
+
+  if (response_id == GTK_RESPONSE_APPLY)
+    return;
+
+  gtk_widget_destroy(widget);
+}
+
+
+static GtkWidget *
+build_ochusha_setting_frame(OchushaApplication *application,
+			    GtkWidget *dialog)
+{
+  GtkWidget *ochusha_frame = gtk_frame_new(_("Setting of Ochusha"));
+  GtkWidget *outer_vbox = gtk_vbox_new(FALSE, 0);
+  GtkWidget *proxy_frame;
+  GtkWidget *proxy_vbox;
+  GtkWidget *hbox;
+  GtkWidget *tmp_widget;
+
+  hbox = gtk_hbox_new(FALSE, 0);
+
+  tmp_widget = gtk_label_new(_("External Web Browser:"));
+  gtk_widget_show(tmp_widget);
+  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, FALSE, FALSE, 5);
+
+  tmp_widget = gtk_entry_new();
+  gtk_entry_set_text(GTK_ENTRY(tmp_widget), application->web_browser_template);
+  gtk_widget_show(tmp_widget);
+  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, TRUE, TRUE, 5);
+  g_object_set_data(G_OBJECT(dialog), "web_browser_template",
+		    tmp_widget);
+
+  gtk_widget_show(hbox);
+  gtk_box_pack_start(GTK_BOX(outer_vbox), hbox, FALSE, FALSE, 5);
+
+  hbox = gtk_hbox_new(FALSE, 0);
+
+  tmp_widget = gtk_label_new(_("(%s will be replaced with URI)"));
+  gtk_widget_show(tmp_widget);
+  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, FALSE, FALSE, 100);
+
+  gtk_widget_show(hbox);
+  gtk_box_pack_start(GTK_BOX(outer_vbox), hbox, FALSE, FALSE, 0);
+
+  gtk_frame_set_shadow_type(GTK_FRAME(ochusha_frame), GTK_SHADOW_ETCHED_OUT);
+  gtk_widget_show(outer_vbox);
+  gtk_container_add(GTK_CONTAINER(ochusha_frame), outer_vbox);
+
+  tmp_widget = gtk_check_button_new_with_label(_("Enable Tooltips of Board Tab Label."));
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp_widget),
+			       application->board_tab_enable_tooltips);
+  gtk_widget_show(tmp_widget);
+  gtk_box_pack_start(GTK_BOX(outer_vbox), tmp_widget, FALSE, FALSE, 5);
+  g_object_set_data(G_OBJECT(dialog), "board_tab_enable_tooltips",
+		    tmp_widget);
+
+#if 0
+  tmp_widget = gtk_check_button_new_with_label(_("Place Boardlist and Threadlist Vertically."));
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp_widget),
+			       application->boardlist_pane_style
+			       == VPANED_STYLE);
+  gtk_widget_show(tmp_widget);
+  gtk_box_pack_start(GTK_BOX(outer_vbox), tmp_widget, FALSE, FALSE, 5);
+  g_object_set_data(G_OBJECT(dialog), "boardlist_pane_style",
+		    tmp_widget);
+#endif
+
+  tmp_widget = gtk_check_button_new_with_label(_("Place Threadlist and Thread Vertically."));
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp_widget),
+			       application->threadlist_pane_style
+			       == VPANED_STYLE);
+  gtk_widget_show(tmp_widget);
+  gtk_box_pack_start(GTK_BOX(outer_vbox), tmp_widget, FALSE, FALSE, 5);
+  g_object_set_data(G_OBJECT(dialog), "threadlist_pane_style",
+		    tmp_widget);
+
+  proxy_frame = gtk_frame_new(_("Proxy Setting"));
+  proxy_vbox = gtk_vbox_new(FALSE, 0);
+  gtk_widget_show(proxy_vbox);
+  gtk_container_add(GTK_CONTAINER(proxy_frame), proxy_vbox);
+
+  tmp_widget = gtk_label_new(_("WARNING: Use of PROXY never tested.  If you know whether this works well or not, please report that."));
+  gtk_widget_show(tmp_widget);
+  gtk_box_pack_start(GTK_BOX(proxy_vbox), tmp_widget, FALSE, FALSE, 5);
+
+  hbox = gtk_hbox_new(FALSE, 0);
+
+  tmp_widget = gtk_radio_button_new_with_label(NULL, _("Don't Use Proxy"));
+  gtk_widget_show(tmp_widget);
+  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, FALSE, FALSE, 0);
+
+  tmp_widget
+    = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(tmp_widget),
+						  _("Use Proxy"));
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp_widget),
+			       application->config.enable_proxy);
+  gtk_widget_show(tmp_widget);
+  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, FALSE, FALSE, 0);
+  g_object_set_data(G_OBJECT(dialog), "enable_proxy", tmp_widget);
+
+  tmp_widget
+    = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(tmp_widget),
+						  _("Use Proxy Only for Posting"));
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp_widget),
+			       application->config.enable_proxy_only_for_posting);
+  gtk_widget_show(tmp_widget);
+  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, FALSE, FALSE, 0);
+  g_object_set_data(G_OBJECT(dialog), "enable_proxy_only_for_posting",
+		    tmp_widget);
+
+  gtk_widget_show(hbox);
+  gtk_box_pack_start(GTK_BOX(proxy_vbox), hbox, FALSE, FALSE, 0);
+
+
+
+  hbox = gtk_hbox_new(FALSE, 0);
+
+  tmp_widget = gtk_label_new(_("Proxy URL:"));
+  gtk_widget_show(tmp_widget);
+  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, FALSE, FALSE, 5);
+
+  tmp_widget = gtk_entry_new();
+  if (application->config.proxy_url != NULL)
+    gtk_entry_set_text(GTK_ENTRY(tmp_widget), application->config.proxy_url);
+  gtk_widget_show(tmp_widget);
+  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, TRUE, TRUE, 5);
+  g_object_set_data(G_OBJECT(dialog), "proxy_url", tmp_widget);
+
+  gtk_widget_show(hbox);
+  gtk_box_pack_start(GTK_BOX(proxy_vbox), hbox, FALSE, FALSE, 0);
+
+
+  hbox = gtk_hbox_new(FALSE, 0);
+
+  tmp_widget = gtk_label_new(_("Proxy User:"));
+  gtk_widget_show(tmp_widget);
+  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, FALSE, FALSE, 5);
+
+  tmp_widget = gtk_entry_new();
+  if (application->config.proxy_user != NULL)
+    gtk_entry_set_text(GTK_ENTRY(tmp_widget), application->config.proxy_user);
+  gtk_widget_show(tmp_widget);
+  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, TRUE, TRUE, 5);
+  g_object_set_data(G_OBJECT(dialog), "proxy_user", tmp_widget);
+
+  tmp_widget = gtk_label_new(_("Proxy Password:"));
+  gtk_widget_show(tmp_widget);
+  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, FALSE, FALSE, 5);
+
+  tmp_widget = gtk_entry_new();
+  if (application->config.proxy_password != NULL)
+    gtk_entry_set_text(GTK_ENTRY(tmp_widget),
+		       application->config.proxy_password);
+  gtk_widget_show(tmp_widget);
+  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, TRUE, TRUE, 5);
+  g_object_set_data(G_OBJECT(dialog), "proxy_password", tmp_widget);
+
+  gtk_widget_show(hbox);
+  gtk_box_pack_start(GTK_BOX(proxy_vbox), hbox, FALSE, FALSE, 5);
+
+
+  gtk_widget_show(proxy_frame);
+  gtk_box_pack_end(GTK_BOX(outer_vbox), proxy_frame, FALSE, FALSE, 5);
+
+  return ochusha_frame;
+}
+
+
+static void
+restore_default_cb(ThreadlistFilterSetting *setting,
+		   OchushaApplication *application)
+{
+  threadlist_filter_setting_set_filter(setting,
+				       &application->user_default_filter);
+
+  setting->filter.ignore_threshold
+    = application->user_default_filter.ignore_threshold;
+
+}
+
+
+static GtkWidget *
+build_threadlist_view_setting_frame(OchushaApplication *application,
+				    GtkWidget *dialog)
+{
+  GtkWidget *threadlist_frame = gtk_frame_new(_("Setting of Threadlist View"));
+  GtkWidget *filter_frame
+    = gtk_frame_new(_("Default Setting of Threadlist's Filter"));
+  GtkWidget *outer_vbox = gtk_vbox_new(FALSE, 0);
+  GtkWidget *vbox;
+  GtkWidget *tmp_widget;
+
+  gtk_frame_set_shadow_type(GTK_FRAME(threadlist_frame),
+			    GTK_SHADOW_ETCHED_OUT);
+  gtk_widget_show(outer_vbox);
+  gtk_container_add(GTK_CONTAINER(threadlist_frame), outer_vbox);
+
+  tmp_widget = gtk_check_button_new_with_label(_("Enable Popup of Thread Title."));
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp_widget),
+			       application->enable_popup_title);
+  gtk_widget_show(tmp_widget);
+  gtk_box_pack_start(GTK_BOX(outer_vbox), tmp_widget, FALSE, FALSE, 0);
+  g_object_set_data(G_OBJECT(dialog), "enable_popup_title",
+		    tmp_widget);
+
+  tmp_widget = gtk_check_button_new_with_label(_("Enable Tooltips of Thread Tab Label."));
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp_widget),
+			       application->thread_tab_enable_tooltips);
+  gtk_widget_show(tmp_widget);
+  gtk_box_pack_start(GTK_BOX(outer_vbox), tmp_widget, FALSE, FALSE, 0);
+  g_object_set_data(G_OBJECT(dialog), "thread_tab_enable_tooltips",
+		    tmp_widget);
+
+  tmp_widget = gtk_check_button_new_with_label(_("Show Threadlist Entry for Selected Thread."));
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp_widget),
+			       application->show_threadlist_entry_when_thread_selected);
+  gtk_widget_show(tmp_widget);
+  gtk_box_pack_start(GTK_BOX(outer_vbox), tmp_widget, FALSE, FALSE, 0);
+  g_object_set_data(G_OBJECT(dialog), "show_threadlist_entry",
+		    tmp_widget);
+
+  gtk_frame_set_shadow_type(GTK_FRAME(filter_frame), GTK_SHADOW_ETCHED_OUT);
+  vbox = gtk_vbox_new(FALSE, 0);
+  gtk_widget_show(vbox);
+  gtk_container_add(GTK_CONTAINER(filter_frame), vbox);
+
+  tmp_widget = threadlist_filter_setting_new_with_initial_setting(&application->user_default_filter);
+  g_signal_connect(G_OBJECT(tmp_widget), "restore_default",
+		   G_CALLBACK(restore_default_cb), application);
+
+  g_object_set_data(G_OBJECT(dialog), "filter_setting", tmp_widget);
+  gtk_widget_show(tmp_widget);
+  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
+  gtk_widget_show(filter_frame);
+
+  gtk_box_pack_start(GTK_BOX(outer_vbox), filter_frame, FALSE, FALSE, 5);
+
+  return threadlist_frame;
+}
+
+
+static GtkWidget *
+build_thread_view_setting_frame(OchushaApplication *application,
+				GtkWidget *dialog)
+{
+  GtkWidget *outer_frame = gtk_frame_new(_("Setting of Thread View"));
+  GtkWidget *outer_vbox = gtk_vbox_new(FALSE, 0);
+  GtkWidget *thread_frame;
+  GtkWidget *tmp_widget;
+  GtkWidget *vbox;
+  GtkWidget *hbox;
+  GtkObject *adjustment;
+
+  gtk_frame_set_shadow_type(GTK_FRAME(outer_frame), GTK_SHADOW_ETCHED_OUT);
+  gtk_widget_show(outer_vbox);
+  gtk_container_add(GTK_CONTAINER(outer_frame), outer_vbox);
+
+  tmp_widget = gtk_check_button_new_with_label(_("Enable Popup of Response."));
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp_widget),
+			       application->enable_popup_response);
+  gtk_widget_show(tmp_widget);
+  gtk_box_pack_start(GTK_BOX(outer_vbox), tmp_widget, FALSE, FALSE, 0);
+  g_object_set_data(G_OBJECT(dialog), "enable_popup_response",
+		    tmp_widget);
+
+  hbox = gtk_hbox_new(FALSE, 0);
+  tmp_widget = gtk_label_new(_("Delay to Show Response Popup:"));
+  gtk_widget_show(tmp_widget);
+  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, FALSE, FALSE, 5);
+
+  adjustment = gtk_adjustment_new(application->popup_response_delay,
+				  0.0, 10000.0, 100.0, 1000.0, 1000.0);
+  tmp_widget = gtk_spin_button_new((GtkAdjustment *)adjustment, 100.0, 0);
+  gtk_widget_show(tmp_widget);
+  g_object_set_data(G_OBJECT(dialog), "popup_response_delay",
+		    tmp_widget);
+  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, FALSE, FALSE, 0);
+
+  tmp_widget = gtk_label_new(_("(ms)"));
+  gtk_widget_show(tmp_widget);
+  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, FALSE, FALSE, 5);
+
+  gtk_widget_show(hbox);
+  gtk_box_pack_start(GTK_BOX(outer_vbox), hbox, FALSE, FALSE, 0);
+  
+  hbox = gtk_hbox_new(FALSE, 0);
+  tmp_widget = gtk_label_new(_("Delay to Close Response Popup:"));
+  gtk_widget_show(tmp_widget);
+  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, FALSE, FALSE, 5);
+
+  adjustment = gtk_adjustment_new(application->popup_close_delay,
+				  0.0, 10000.0, 100.0, 1000.0, 1000.0);
+  tmp_widget = gtk_spin_button_new((GtkAdjustment *)adjustment, 100.0, 0);
+  gtk_widget_show(tmp_widget);
+  g_object_set_data(G_OBJECT(dialog), "popup_close_delay",
+		    tmp_widget);
+  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, FALSE, FALSE, 0);
+
+  tmp_widget = gtk_label_new(_("(ms)"));
+  gtk_widget_show(tmp_widget);
+  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, FALSE, FALSE, 5);
+
+  gtk_widget_show(hbox);
+  gtk_box_pack_start(GTK_BOX(outer_vbox), hbox, FALSE, FALSE, 0);
+
+
+
+  thread_frame = gtk_frame_new(_("Default Setting of Thread Views"));
+  gtk_frame_set_shadow_type(GTK_FRAME(thread_frame), GTK_SHADOW_ETCHED_OUT);
+
+  vbox = gtk_vbox_new(FALSE, 0);
+
+  gtk_widget_show(vbox);
+  gtk_container_add(GTK_CONTAINER(thread_frame), vbox);
+
+  tmp_widget = gtk_check_button_new_with_label(_("Show mailto literally."));
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp_widget),
+			       application->show_mailto_mode
+			       == THREAD_VIEW_MAILTO_MODE_SHOW);
+  gtk_widget_show(tmp_widget);
+  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 5);
+  g_object_set_data(G_OBJECT(dialog), "show_mailto_literally",
+		    tmp_widget);
+
+  gtk_widget_show(thread_frame);
+  gtk_box_pack_start(GTK_BOX(outer_vbox), thread_frame, TRUE, TRUE, 5);
+
+  return outer_frame;
+}
+
+
+static void
+open_preference_dialog(OchushaApplication *application)
+{
+  MAIN_UI_LOCK
+  {
+    if (preference_dialog == NULL)
+      {
+	GtkWidget *notebook;
+	GtkWidget *tmp_widget;
+	preference_dialog
+	  = gtk_dialog_new_with_buttons(_("Preferences of Ochusha"),
+					application->top_level,
+					GTK_DIALOG_DESTROY_WITH_PARENT,
+					GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					GTK_STOCK_APPLY, GTK_RESPONSE_APPLY,
+					GTK_STOCK_OK, GTK_RESPONSE_OK,
+					NULL);
+	gtk_dialog_set_default_response(GTK_DIALOG(preference_dialog),
+					GTK_RESPONSE_OK);
+	g_signal_connect(G_OBJECT(preference_dialog), "response",
+			 G_CALLBACK(preference_dialog_response_cb),
+			 application);
+	g_signal_connect(G_OBJECT(preference_dialog), "destroy",
+			 G_CALLBACK(gtk_widget_destroyed),
+			 &preference_dialog);
+
+	notebook = gtk_notebook_new();
+	gtk_widget_show(notebook);
+	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(preference_dialog)->vbox),
+			  notebook);
+
+	tmp_widget = build_ochusha_setting_frame(application,
+						 preference_dialog);
+	gtk_widget_show(tmp_widget);
+	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), tmp_widget,
+				 gtk_label_new(_("Setting of Ochusha")));
+
+	tmp_widget = build_threadlist_view_setting_frame(application,
+							 preference_dialog);
+	gtk_widget_show(tmp_widget);
+	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), tmp_widget,
+				 gtk_label_new(_("Setting of Threadlist View")));
+
+	tmp_widget = build_thread_view_setting_frame(application,
+						     preference_dialog);
+	gtk_widget_show(tmp_widget);
+	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), tmp_widget,
+				 gtk_label_new(_("Setting of Thread View")));
+
+	gtk_widget_show(preference_dialog);
+      }
+  }
+  MAIN_UI_UNLOCK;
+}
+
+
 static void
 open_about_dialog(OchushaApplication *application)
 {
-  /* about_dialog */
   MAIN_UI_LOCK
     {
       if (about_dialog == NULL)
@@ -1345,6 +1991,9 @@ open_about_dialog(OchushaApplication *application)
 	  g_signal_connect(G_OBJECT(about_dialog), "response",
 			   G_CALLBACK(about_dialog_response_cb),
 			   NULL);
+	  g_signal_connect(G_OBJECT(about_dialog), "destroy",
+			   G_CALLBACK(gtk_widget_destroyed),
+			   &about_dialog);
 
 	  snprintf(text_buffer, 1024, "%s %s %s",
 		   _("ochusha"), _("Version"), PACKAGE_VERSION);
@@ -1383,18 +2032,10 @@ open_about_dialog(OchushaApplication *application)
 static void
 about_dialog_response_cb(GtkWidget *widget, gint response_id, gpointer unused)
 {
-  GtkWidget *dialog;
-  MAIN_UI_LOCK
-    {
-      dialog = about_dialog;
-      about_dialog = NULL;
-    }
-  MAIN_UI_UNLOCK;
-
-  if (dialog == NULL)
+  if (widget == NULL)
     return;
 
-  gtk_widget_destroy(dialog);
+  gtk_widget_destroy(widget);
 }
 
 
