@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_ui.h,v 1.22 2003/11/23 22:04:09 fuyu Exp $
+ * $Id: ochusha_ui.h,v 1.23 2003/11/25 18:19:23 fuyu Exp $
  */
 
 #ifndef _OCHUSHA_UI_H_
@@ -197,6 +197,8 @@ struct _OchushaApplication
 
 
   /* ɽϢ */
+  char *thread_view_font_name;
+
   /* ΤΥǥե */
   /* ¾ĤΥǥեȤȥ̤Ѱդͽꡣ */
   ThreadViewMailtoMode show_mailto_mode;
@@ -247,6 +249,8 @@ typedef enum
   MAIN_MENU_REFRESH_THREADLIST,
   MAIN_MENU_REFRESH_THREADLIST_ALL,
   MAIN_MENU_HIDE_HIDDEN_THREADS,
+  MAIN_MENU_DEBUG_CONSOLE,
+  MAIN_MENU_OPTIONS,
   MAIN_MENU_ABOUT,
 } OchushaMainMenuAction;
 
