@@ -23,12 +23,15 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: text_search_window.c,v 1.5 2003/05/17 18:59:11 fuyu Exp $
+ * $Id: text_search_window.c,v 1.7 2003/06/07 14:35:31 fuyu Exp $
  */
 
-#include "ochusha.h"
-#include "ochusha_ui.h"
+#include "config.h"
 
+#include "ochusha_private.h"
+#include "ochusha.h"
+
+#include "ochusha_ui.h"
 #include "text_search_window.h"
 
 #include "marshal.h"
@@ -372,7 +375,7 @@ text_search_window_finalize(GObject *object)
   TextSearchWindow *window = TEXT_SEARCH_WINDOW(object);
 #endif
 #if DEBUG_WIDGET_MOST
-  fprintf(stderr, "text_search_window_finalize(0x%x)\n", (int)object);
+  fprintf(stderr, "text_search_window_finalize(%p)\n", object);
 #endif
 
   if (G_OBJECT_CLASS(parent_class)->finalize)
@@ -385,12 +388,12 @@ text_search_window_destroy(GtkObject *object)
 {
   TextSearchWindow *window = TEXT_SEARCH_WINDOW(object);
 #if DEBUG_WIDGET_MOST
-  fprintf(stderr, "text_search_window_destroy(0x%x)\n", (int)object);
+  fprintf(stderr, "text_search_window_destroy(%p)\n", object);
 #endif
 
   if (window->key != NULL)
     {
-      g_free(window->key);
+      G_FREE(window->key);
       window->key = NULL;
     }
 
@@ -444,8 +447,8 @@ text_search_window_update_query(TextSearchWindow *window)
   if (window->key == NULL || (key != NULL && strcmp(window->key, key) != 0))
     {
       if (window->key != NULL)
-	g_free(window->key);
-      window->key = g_strdup(key);
+	G_FREE(window->key);
+      window->key = G_STRDUP(key);
 #if DEBUG_WIDGET
       fprintf(stderr, "new key: %s\n", window->key);
 #endif
