@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: threadlist_filter.c,v 1.6 2003/06/07 14:35:31 fuyu Exp $
+ * $Id: threadlist_filter.c,v 1.7 2003/11/23 22:04:09 fuyu Exp $
  */
 
 #include "config.h"
@@ -102,7 +102,8 @@ static void text_set_cell_data_applied_rule(GtkTreeViewColumn *column,
 					    GtkTreeIter *iter,
 					    ThreadlistFilterSetting *data);
 
-static const gchar *threadlist_filter_rule_ochusha_default = "FrhSNsa";
+static const gchar *threadlist_filter_rule_ochusha_default
+  = OCHUSHA_DEFAULT_THREADLIST_FILTER_RULE;
 static gchar *threadlist_filter_rule_default = NULL;
 static int threadlist_filter_rule_default_ignore_threshold;
 
