@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: threadlist_view.c,v 1.19 2003/11/07 15:52:40 fuyu Exp $
+ * $Id: threadlist_view.c,v 1.21 2003/11/23 22:39:35 fuyu Exp $
  */
 
 #include "config.h"
@@ -141,7 +141,7 @@ threadlist_view_get_type(void)
 
 
 enum {
-  OPEN_VIEW_SIGNAL,
+  OPEN_THREAD_VIEW_SIGNAL,
 
   THREAD_MOUSE_OVER_SIGNAL,
   THREAD_MOUSE_OUT_SIGNAL,
@@ -174,6 +174,7 @@ enum {
 enum {
   POPUP_MENU_OPEN_SELECTION = 1,
   POPUP_MENU_OPEN_SELECTION_IN_TAB,
+  POPUP_MENU_OPEN_SELECTION_WITH_BROWSER,
   POPUP_MENU_MARK_SELECTION,
   POPUP_MENU_UNMARK_SELECTION,
   POPUP_MENU_TOGGLE_HIDE_SELECTION,
@@ -276,12 +277,19 @@ threadlist_view_class_init(ThreadlistViewClass *klass)
 	NULL					/* type */
       },
       {
-	_("/Open Selected Threads In _Tab"),	/* menu path */
+	_("/Open Selected Threads in _Tab"),	/* menu path */
 	NULL,					/* accelerator */
 	popup_menu_callback,			/* callback_func */
 	POPUP_MENU_OPEN_SELECTION_IN_TAB,	/* act */
 	NULL					/* type */
       },
+      {
+	_("/Open Selected Threads with Web Broser"),/* menu path */
+	NULL,					/* accelerator */
+	popup_menu_callback,			/* callback_func */
+	POPUP_MENU_OPEN_SELECTION_WITH_BROWSER,	/* act */
+	NULL					/* type */
+      },
       {
 	_("/-------"),
 	NULL,
@@ -346,15 +354,16 @@ threadlist_view_class_init(ThreadlistViewClass *klass)
   /* GtkTreeView signals */
   tree_view_class->move_cursor = threadlist_view_move_cursor;
 
-  threadlist_view_signals[OPEN_VIEW_SIGNAL] =
-    g_signal_new("open_view",
+  threadlist_view_signals[OPEN_THREAD_VIEW_SIGNAL] =
+    g_signal_new("open_thread_view",
 		 G_TYPE_FROM_CLASS(klass),
 		 G_SIGNAL_RUN_LAST,
-		 G_STRUCT_OFFSET(ThreadlistViewClass, open_view),
+		 G_STRUCT_OFFSET(ThreadlistViewClass, open_thread_view),
 		 NULL, NULL,
-		 ochusha_marshal_VOID__POINTER_BOOLEAN,
-		 G_TYPE_NONE, 2,
-		 G_TYPE_POINTER,
+		 ochusha_marshal_VOID__OBJECT_BOOLEAN_BOOLEAN,
+		 G_TYPE_NONE, 3,
+		 OCHUSHA_TYPE_BBS_THREAD,
+		 G_TYPE_BOOLEAN,
 		 G_TYPE_BOOLEAN);
 
   threadlist_view_signals[THREAD_MOUSE_OVER_SIGNAL] =
@@ -545,7 +554,7 @@ threadlist_view_class_init(ThreadlistViewClass *klass)
 			       G_TYPE_INT,
 			       THREADLIST_SEARCH_ACTION_TERMINATE);
 
-  klass->open_view = NULL;
+  klass->open_thread_view = NULL;
 
   klass->thread_mouse_over = NULL;
   klass->thread_mouse_out = NULL;
@@ -1095,10 +1104,20 @@ do_on_selection_helper(GtkTreeModel *model, GtkTreePath *path,
       /* FALL THROUGH */
     case POPUP_MENU_OPEN_SELECTION_IN_TAB:
       g_signal_emit(G_OBJECT(args->view),
-		    threadlist_view_signals[OPEN_VIEW_SIGNAL],
+		    threadlist_view_signals[OPEN_THREAD_VIEW_SIGNAL],
 		    0,
 		    thread,
-		    what_to_do == POPUP_MENU_OPEN_SELECTION_IN_TAB);
+		    what_to_do == POPUP_MENU_OPEN_SELECTION_IN_TAB,
+		    FALSE);
+      break;
+
+    case POPUP_MENU_OPEN_SELECTION_WITH_BROWSER:
+      g_signal_emit(G_OBJECT(args->view),
+		    threadlist_view_signals[OPEN_THREAD_VIEW_SIGNAL],
+		    0,
+		    thread,
+		    FALSE,
+		    TRUE);
       break;
 
     case POPUP_MENU_MARK_SELECTION:
@@ -1350,10 +1369,11 @@ tree_view_button_press_event_cb(GtkWidget *widget, GdkEventButton *event,
 	  else
 	    {
 	      g_signal_emit(G_OBJECT(view),
-			    threadlist_view_signals[OPEN_VIEW_SIGNAL],
+			    threadlist_view_signals[OPEN_THREAD_VIEW_SIGNAL],
 			    0,
 			    data,
-			    event->button == 2);
+			    event->button == 2,
+			    FALSE);
 	    }
 	}
     }
@@ -1569,6 +1589,40 @@ decorate_threadlist_entry(GtkTreeViewColumn *tree_column,
 }
 
 
+static void
+text_set_cell_data_rank(GtkTreeViewColumn *tree_column,
+			GtkCellRenderer *renderer,
+			GtkTreeModel *tree_model, GtkTreeIter *iter,
+			gpointer view)
+{
+  GValue src = { 0, };
+  GValue value = { 0, };
+  int rank;
+
+  g_return_if_fail(gtk_list_store_iter_is_valid(GTK_LIST_STORE(tree_model),
+						iter));
+
+  gtk_tree_model_get_value(tree_model, iter, THREADLIST_RANK, &src);
+  rank = g_value_get_int(&src);
+
+  g_value_init(&value, G_TYPE_STRING);
+  if (rank <= 0)
+    {
+      g_value_set_static_string(&value, _("Dead"));
+    }
+  else
+    {
+      /* ̾ */
+      gchar buffer[512];
+      snprintf(buffer, 512, "%d", rank);
+      g_value_set_string(&value, buffer);
+    }
+
+  g_object_set_property((GObject *)renderer, "text", &value);
+  decorate_threadlist_entry(tree_column, renderer, tree_model, iter, view);
+}
+
+
 static void
 active_set_cell_data_flags(GtkTreeViewColumn *tree_column,
 			   GtkCellRenderer *renderer, GtkTreeModel *tree_model,
@@ -1703,7 +1757,7 @@ threadlist_view_append_rank(ThreadlistView *view)
 						    "text", THREADLIST_RANK,
 						    NULL);
   gtk_tree_view_column_set_cell_data_func(column, renderer,
-					  decorate_threadlist_entry,
+					  text_set_cell_data_rank,
 					  view, NULL);
   gtk_tree_view_append_column(&view->view, column);
   view->rank_column = column;
@@ -1959,7 +2013,9 @@ threadlist_view_append_thread(ThreadlistView *view, OchushaBBSThread *thread,
   gtk_list_store_append(GTK_LIST_STORE(view->model), &info->iter);
   info->threadlist_view = view;
   set_thread_values(view, GTK_LIST_STORE(view->model),
-		    last_rank, new_rank, thread);
+		    last_rank,
+		    ((thread->flags & OCHUSHA_BBS_THREAD_DAT_DROPPED) == 0
+		     ? new_rank : RANK_DAT_DROPPED), thread);
 }
 
 
