@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: htmlutils.c,v 1.6 2003/11/18 21:02:05 fuyu Exp $
+ * $Id: htmlutils.c,v 1.7 2003/11/26 04:13:51 fuyu Exp $
  */
 
 #include "config.h"
@@ -59,8 +59,9 @@ static gchar *encode_unicode_to_utf8(unsigned int character, gchar *buffer);
 
 
 static GHashTable *default_entity_dictionary = NULL;
+static GHashTable *reverse_entity_dictionary = NULL;
 
-static gchar *default_entry[] =
+static gchar *default_entity[] =
   {
     /* <!ENTITY % HTMLlat1 PUBLIC "-//W3C//ENTITIES// Latin 1//EN//HTML"> */
 
@@ -353,18 +354,21 @@ static gchar *default_entry[] =
 void
 initialize_default_entity_dictionary()
 {
-  gchar **entry = default_entry;
+  gchar **entry = default_entity;
 
   if (default_entity_dictionary != NULL)
     return;
 
   default_entity_dictionary = g_hash_table_new(g_str_hash, g_str_equal);
+  reverse_entity_dictionary = g_hash_table_new(g_str_hash, g_str_equal);
 
   /* ǤäϿ롣*/
   while (*entry != NULL)
     {
       g_hash_table_insert(default_entity_dictionary,
 			  entry[0], entry[1]);
+      g_hash_table_insert(reverse_entity_dictionary,
+			  entry[1], entry[0]);
       entry += 2;
     }
 }
@@ -377,6 +381,13 @@ get_default_entity_dictionary()
 }
 
 
+GHashTable *
+get_reverse_entity_dictionary()
+{
+  return reverse_entity_dictionary;
+}
+
+
 #define UTF8_BUFFER_SIZE	4
 static gboolean
 default_entity_ref_handler(ElementHandler *handler, void *context,
