@@ -33,7 +33,10 @@ extern void libochusha_marshal_BOOLEAN__OBJECT_INT_STRING (GClosure     *closure
 /* VOID:POINTER (marshal.list:4) */
 #define libochusha_marshal_VOID__POINTER	g_cclosure_marshal_VOID__POINTER
 
-/* VOID:OBJECT,POINTER (marshal.list:5) */
+/* VOID:STRING (marshal.list:5) */
+#define libochusha_marshal_VOID__STRING	g_cclosure_marshal_VOID__STRING
+
+/* VOID:OBJECT,POINTER (marshal.list:6) */
 extern void libochusha_marshal_VOID__OBJECT_POINTER (GClosure     *closure,
                                                      GValue       *return_value,
                                                      guint         n_param_values,
