@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_bbs_table.c,v 1.13 2003/11/20 03:14:37 fuyu Exp $
+ * $Id: ochusha_bbs_table.c,v 1.14 2003/11/21 21:30:03 fuyu Exp $
  */
 
 #include "config.h"
@@ -31,9 +31,10 @@
 #include "ochusha_private.h"
 #include "ochusha.h"
 #include "ochusha_bbs_table.h"
+#include "ochusha_board_2ch.h"
 #include "ochusha_board_category.h"
 #include "ochusha_bulletin_board.h"
-#include "ochusha_board_2ch.h"
+#include "ochusha_utils_2ch.h"
 
 #include "marshal.h"
 
@@ -47,29 +48,6 @@
 #include <string.h>
 #include <unistd.h>
 
-#if ENABLE_REGEXP
-# if HAVE_ONIGURUMA
-#  if HAVE_ONIG_ONIGPOSIX_H
-#    include <onig/onigposix.h>
-#  elif HAVE_ONIGPOSIX_H
-#    include <onigposix.h>
-#  else
-#    error "Use of oniguruma without onig/onigposix.h header isn't considered."
-#  endif
-# else
-#  if HAVE_POSIX_REGEX
-#    if HAVE_SYS_TYPES_H
-#      include <sys/types.h>
-#    endif
-#    if HAVE_REGEX_H
-#      include <regex.h>
-#    else
-#      warning "I don't know suitable header file of your posix regex library."
-#    endif
-#  endif
-# endif
-#endif
-
 
 static void ochusha_bbs_table_class_init(OchushaBBSTableClass *klass);
 static void ochusha_bbs_table_init(OchushaBBSTable *table);
@@ -117,10 +95,6 @@ enum {
 static GObjectClass *parent_class = NULL;
 static gint bbs_table_signals[LAST_SIGNAL] = { 0, 0, 0, 0 };
 
-#if ENABLE_REGEXP
-static regex_t cgi_query_pattern;
-#endif
-
 static void
 ochusha_bbs_table_class_init(OchushaBBSTableClass *klass)
 {
@@ -178,11 +152,6 @@ ochusha_bbs_table_class_init(OchushaBBSTableClass *klass)
   klass->boardlist_write_category_element = NULL;
   klass->boardlist_read_board_element = NULL;
   klass->boardlist_write_board_element = NULL;
-
-#if ENABLE_REGEXP
-  if (regcomp(&cgi_query_pattern, "&?([^=]+)=([^&]*)", REG_EXTENDED))
-    fprintf(stderr, "invalid regular expression\n");
-#endif
 }
 
 
@@ -340,200 +309,6 @@ ochusha_bbs_table_lookup_board_by_name(OchushaBBSTable *table,
 }
 
 
-static gboolean
-parse_thread_path(char *path, char **thread_id_p,
-		  unsigned int *from_p, unsigned int *to_p)
-{
-  char *thread_id = NULL;
-  unsigned int from = 0;
-  unsigned int to = 0;
-  char *slash_pos = strchr(path + 1, '/');
-
-#if 0
-  fprintf(stderr, "parse_thread_path()\n");
-  fprintf(stderr, "  path: %s\n", path);
-#endif
-
-  if (slash_pos != NULL)
-    {
-      if (thread_id_p != NULL)
-	thread_id = G_STRNDUP(path, slash_pos - path);
-      else
-	thread_id = path;
-      path = slash_pos + 1;
-
-      if (*path != '\0')
-	{
-	  char tmp_char;
-	  char *hyphen_pos = strchr(path, '-');
-	  if (hyphen_pos != NULL)
-	    {
-	      tmp_char = *hyphen_pos;
-	      *hyphen_pos = '\0';
-	    }
-	  else
-	    tmp_char = '\0';
-
-	  if (hyphen_pos != path)
-	    {
-	      if (sscanf(path, "%d", &from) != 1)
-		from = 0;
-	    }
-
-	  if (hyphen_pos != NULL && hyphen_pos[1] != '\0')
-	    {
-	      if (sscanf(hyphen_pos + 1, "%d", &to) != 1)
-		to = from;
-	    }
-	}
-    }
-  else
-    {
-      if (thread_id_p != NULL && *path != '\0')
-	thread_id = G_STRDUP(path);
-      else
-	thread_id = path;
-    }
-
-  if (thread_id_p != NULL)
-    *thread_id_p = thread_id;
-
-  if (from_p != NULL)
-    *from_p = from;
-
-  if (to_p != NULL)
-    *to_p = to;
-
-#if 0
-  fprintf(stderr, "  thread_id: %s\n", thread_id);
-  fprintf(stderr, "  from=%d, to=%d\n", from, to);
-#endif
-
-  return thread_id != NULL;
-}
-
-
-#if ENABLE_REGEXP
-static gboolean
-parse_cgi_query(const char *query, char **board_id_p, char **thread_id_p,
-		unsigned int *from_p, unsigned int *to_p)
-{
-  char board_id[PATH_MAX];
-  char thread_id[PATH_MAX];
-  char tmp_buf[PATH_MAX];
-  unsigned int from = 0;
-  unsigned int to = 0;
-  regmatch_t match[3];
-  char *tmp_pos;
-
-#if 0
-  fprintf(stderr, "parse_cgi_query()\n");
-  fprintf(stderr, "  query: %s\n", query);
-#endif
-
-  board_id[0] = '\0';
-  thread_id[0] = '\0';
-
-  tmp_pos = strchr(query, '?');
-  if (tmp_pos != NULL)
-    query = tmp_pos + 1;	/* ?ޤǤФ */
-
-  while (regexec(&cgi_query_pattern, query, 2, match, 0) == 0)
-    {
-      int match_len = match[0].rm_eo - match[0].rm_so;
-      const char *name_head = query + match[1].rm_so;
-      int name_len = match[1].rm_eo - match[1].rm_so;
-      const char *value_head = query + match[2].rm_so;
-      int value_len = match[2].rm_eo - match[2].rm_so;
-
-      if (g_ascii_strncasecmp("BBS", name_head, name_len) == 0)
-	{
-	  if (value_len < (PATH_MAX - 1))
-	    {
-	      memcpy(board_id, value_head, value_len);
-	      board_id[value_len] = '\0';
-	    }
-	}
-      else if (g_ascii_strncasecmp("KEY", name_head, name_len) == 0)
-	{
-	  if (value_len < (PATH_MAX - 1))
-	    {
-	      memcpy(thread_id, value_head, value_len);
-	      thread_id[value_len] = '\0';
-	    }
-	}
-      else if (g_ascii_strncasecmp("START", name_head, name_len) == 0
-	       || g_ascii_strncasecmp("ST", name_head, name_len) == 0)
-	{
-	  if (value_len > 0 && value_len < (PATH_MAX - 1))
-	    {
-	      memcpy(tmp_buf, value_head, value_len);
-	      tmp_buf[value_len] = '\0';
-	      if (sscanf(tmp_buf, "%d", &from) != 1)
-		from = 0;
-	    }
-	}
-      else if (g_ascii_strncasecmp("END", name_head, name_len) == 0
-	       || g_ascii_strncasecmp("TO", name_head, name_len) == 0)
-	{
-	  if (value_len > 0 && value_len < (PATH_MAX - 1))
-	    {
-	      memcpy(tmp_buf, value_head, value_len);
-	      tmp_buf[value_len] = '\0';
-	      if (sscanf(tmp_buf, "%d", &to) != 1)
-		to = 0;
-	    }
-	}
-
-      query += match_len;
-    }
-
-  if (from == 0 && to != 0)
-    from = to;
-  else if (to == 0 && from != 0)
-    to = from;
-  else if (from == 0 && to == 0)
-    {
-      from = 1;
-      to = 1;
-    }
-
-  if (from_p != NULL)
-    *from_p = from;
-
-  if (to_p != NULL)
-    *to_p = to;
-
-  if (board_id[0] == '\0')
-    thread_id[0] = '\0';
-
-  if (board_id_p != NULL)
-    {
-      if (board_id[0] != '\0')
-	*board_id_p = G_STRDUP(board_id);
-      else
-	*board_id_p = NULL;
-    }
-
-  if (thread_id_p != NULL)
-    {
-      if (thread_id[0] != '\0')
-	*thread_id_p = G_STRDUP(thread_id);
-      else
-	*thread_id_p = NULL;
-    }
-
-#if 0
-  fprintf(stderr, "  board_id: %s\n", board_id);
-  fprintf(stderr, "  thread_id: %s\n", thread_id);
-  fprintf(stderr, "  from=%d, to=%d\n", from, to);
-#endif
-
-  return board_id[0] != '\0';
-}
-#endif
-
-
 /*
  * Ϳ줿URLϤбƤʷǼĤΤΤɤ
  * Ƚꤹ롣
@@ -565,315 +340,26 @@ ochusha_bbs_table_check_url(OchushaBBSTable *table, const char *url,
 			    char **thread_id_p,
 			    unsigned int *from_p, unsigned int *to_p)
 {
-  char *server;
-  char *abs_path;
-  char *path_pos;
-  OchushaBulletinBoard *board = NULL;
-  OchushaBBSType bbs_type = OCHUSHA_BBS_TYPE_UNKNOWN;
-  char *thread_id = NULL;
-  int from = 0;
-  int to = 0;
-  char board_url[PATH_MAX];
-  char tmp_buf[PATH_MAX];
+  char *board_url = NULL;
+  gboolean result;
 
-  g_return_val_if_fail(OCHUSHA_IS_BBS_TABLE(table) && url != NULL, FALSE);
-
-  board_url[0] = '\0';
-
-  server = ochusha_utils_url_extract_http_server(url);
-  abs_path = ochusha_utils_url_extract_http_absolute_path(url);
-
-#if 0
-  fprintf(stderr, "ochusha_bbs_table_check_url()\n");
-  fprintf(stderr, "  url: %s\n", url);
-  fprintf(stderr, "  server: %s\n", server);
-  fprintf(stderr, "  abs_path: %s\n", abs_path);
-#endif
-
-  if (server == NULL || abs_path == NULL || *abs_path != '/')
-    goto done;
-
-  path_pos = abs_path + 1;
-  if (strncmp(path_pos, "test/read.cgi", 13) == 0)
+  if ((result = ochusha_utils_2ch_check_url(url, &board_url, bbs_type_p,
+					    NULL, thread_id_p, from_p, to_p)))
     {
-      bbs_type = OCHUSHA_BBS_TYPE_2CH;
-      if (path_pos[13] == '/')
+      if (board_p != NULL)
 	{
-	  char *slash_pos = strchr(path_pos + 14, '/');
-	  if (slash_pos != NULL)
-	    {
-	      char tmp_char = slash_pos[1];
-	      slash_pos[1] = '\0';
-	      if (snprintf(board_url, PATH_MAX, "http://%s%s", server,
-			   path_pos + 13) < PATH_MAX)
-		{
-#if 0
-		  fprintf(stderr, "  board_url: %s\n", board_url);
-#endif
-		  board = ochusha_bbs_table_lookup_board_by_url(table,
-								board_url);
-		  slash_pos[1] = tmp_char;
-		  parse_thread_path(slash_pos + 1, &thread_id, &from, &to);
-		}
-	      else
-		board_url[0] = '\0';
-	    }
-	}
-#if ENABLE_REGEXP
-      else if (path_pos[13] == '?')
-	{
-	  char *board_id = NULL;
-	  if (parse_cgi_query(path_pos + 14, &board_id, &thread_id,
-			      &from, &to))
-	    {
-	      if (snprintf(board_url, PATH_MAX, "http://%s/%s/", server,
-			   board_id) < PATH_MAX)
-		board = ochusha_bbs_table_lookup_board_by_url(table,
-							      board_url);
-	      else
-		board_url[0] = '\0';
-#if 0
-	      fprintf(stderr, "  board_url: %s\n", board_url);
-#endif
-	    }
-	  if (board_id != NULL)
-	    G_FREE(board_id);
-	}
-#endif
-    }
-  else if (g_ascii_strcasecmp(server, "jbbs.shitaraba.com") == 0)
-    {
-      /* JBBSФ̯ʴ*/
-      char *tmp_pos = strstr(path_pos, "bbs/read.cgi");
-      bbs_type = OCHUSHA_BBS_TYPE_JBBS_SHITARABA;
-      if (tmp_pos == NULL)
-	{
-	  /* URLȤƤ
-	   * http://jbbs.shitaraba.com/computer/351/
-	   * ߤʤΤ⤢롣
-	   */
-	  tmp_pos = strchr(path_pos, '/');
-	  if (tmp_pos != NULL)
-	    tmp_pos = strchr(tmp_pos + 1, '/');
-	  if (tmp_pos != NULL)
-	    {
-	      char tmp_char = tmp_pos[1];
-	      tmp_pos[1] = '\0';
-	      if (snprintf(board_url, PATH_MAX, "http://%s/%s/", server,
-			   path_pos) < PATH_MAX)
-		board = ochusha_bbs_table_lookup_board_by_url(table,
-							      board_url);
-	      else
-		board_url[0] = '\0';
-	      tmp_pos[1] = tmp_char;
-	    }
-	}
-      else
-	{
-	  /*
-	   * URLȤƤϰʲ2ĤηΤʤ¾ˤ⤢뤫⡣
-	   */
-	  if (tmp_pos[12] == '/')
-	    {
-	      /*
-	       * http://jbbs.shitaraba.com/bbs/read.cgi/computer/351/1040452877/109
-	       * URL
-	       */
-	      char *slash_pos = strchr(tmp_pos + 1, '/');
-	      if (slash_pos != NULL)
-		slash_pos = strchr(slash_pos + 1, '/');	/* 2ܤ'/'õ*/
-
-	      if (slash_pos != NULL)
-		{
-		  /* (tmp_pos + 12)slash_posޤǤʸ
-		   * "/computer/351/"ʬˤʤ
-		   */
-		  char tmp_char = slash_pos[1];
-		  slash_pos[1] = '\0';
-		  if (snprintf(board_url, PATH_MAX, "http://%s%s", server,
-			       tmp_pos + 12) < PATH_MAX)
-		    {
-		      board = ochusha_bbs_table_lookup_board_by_url(table,
-								    board_url);
-		      slash_pos[1] = tmp_char;
-		      parse_thread_path(slash_pos + 1, &thread_id, &from, &to);
-		    }
-		  else
-		    board_url[0] = '\0';
-		}
-	    }
-#if ENABLE_REGEXP
-	  else if (tmp_pos[12] == '?')
-	    {
-	      /*
-	       * http://jbbs.shitaraba.com/computer/bbs/read.cgi?BBS=351&KEY=1040452877&START=109&END=109&NOFIRST=TRUE
-	       * URL
-	       */
-	      char *board_id = NULL;
-	      if (parse_cgi_query(tmp_pos + 13, &board_id, &thread_id,
-				  &from, &to))
-		{
-		  *tmp_pos = '\0';
-		  if (snprintf(board_url, PATH_MAX, "http://%s%s%s/",
-			       server, abs_path, board_id) < PATH_MAX)
-		    board = ochusha_bbs_table_lookup_board_by_url(table,
-								  board_url);
-		  else
-		    board_url[0] = '\0';
-		}
-	      if (board_id != NULL)
-		G_FREE(board_id);
-	    }
-#endif
+	  *board_p = ochusha_bbs_table_lookup_board_by_url(table, board_url);
+	  if (*board_p != NULL && bbs_type_p != NULL)
+	    *bbs_type_p = (*board_p)->bbs_type;
 	}
     }
-#if ENABLE_REGEXP
-  else if (strncmp(path_pos, "bbs/read.pl?", 12) == 0)
-    {
-      char *board_id = NULL;
-      bbs_type = OCHUSHA_BBS_TYPE_MACHIBBS;
-      if (parse_cgi_query(path_pos + 12, &board_id, &thread_id,
-			  &from, &to))
-	{
-	  if (snprintf(board_url, PATH_MAX, "http://%s/%s/", server, board_id)
-	      < PATH_MAX)
-	    board = ochusha_bbs_table_lookup_board_by_url(table, board_url);
-	  else
-	    board_url[0] = '\0';
-#if 0
-	  fprintf(stderr, "  board_url: %s\n", board_url);
-#endif
-	}
-      if (board_id != NULL)
-	G_FREE(board_id);
-    }
-  else if (strncmp(path_pos, "read.cgi?", 9) == 0)
-    {
-      char *board_id = NULL;
-      bbs_type = OCHUSHA_BBS_TYPE_MITINOKU;
-      if (parse_cgi_query(path_pos + 12, &board_id, &thread_id,
-			  &from, &to))
-	{
-	  if (snprintf(board_url, PATH_MAX, "http://%s/%s/", server, board_id)
-	      < PATH_MAX)
-	    board = ochusha_bbs_table_lookup_board_by_url(table, board_url);
-	  else
-	    board_url[0] = '\0';
-#if 0
-	  fprintf(stderr, "  board_url: %s\n", board_url);
-#endif
-	}
-      if (board_id != NULL)
-	G_FREE(board_id);
-    }
-#endif
-  else if (strncmp(path_pos, "bbs/read.cgi", 12) == 0)
-    {
-      if (path_pos[12] == '/')
-	{
-	  char *slash_pos = strchr(path_pos + 13, '/');
-	  if (slash_pos != NULL)
-	    {
-	      int len = slash_pos - (path_pos + 12);
-	      memcpy(tmp_buf, path_pos + 12, len);
-	      tmp_buf[len] = '\0';
-	      if (snprintf(board_url, PATH_MAX, "http://%s%s", server,
-			   tmp_buf) < PATH_MAX)
-		{
-		  board = ochusha_bbs_table_lookup_board_by_url(table,
-								board_url);
-		  parse_thread_path(slash_pos + 1, &thread_id, &from, &to);
-		  bbs_type = OCHUSHA_BBS_TYPE_JBBS;	/* ̤ƥ */
-		}
-	      else
-		board_url[0] = '\0';
-#if 0
-	      fprintf(stderr, "  board_url: %s\n", board_url);
-#endif
-	    }
-	}
-#if ENABLE_REGEXP
-      else if (path_pos[12] == '?')
-	{
-	  char *board_id = NULL;
-	  if (parse_cgi_query(path_pos + 13, &board_id, &thread_id,
-			      &from, &to))
-	    {
-	      if (snprintf(board_url, PATH_MAX, "http://%s/%s/", server,
-			   board_id) < PATH_MAX)
-		board = ochusha_bbs_table_lookup_board_by_url(table,
-							      board_url);
-	      else
-		board_url[0] = '\0';
-#if 0
-	      fprintf(stderr, "  board_url: %s\n", board_url);
-#endif
-	    }
-	  if (board_id != NULL)
-	    G_FREE(board_id);
-	}
-#endif
-    }
-  else
-    {
-      /* 2chäݤġġ*/
-      char *slash_pos = strchr(path_pos, '/');
-      if (slash_pos != NULL)
-	{
-	  char tmp_char = slash_pos[1];
-	  slash_pos[1] = '\0';
-	  snprintf(board_url, PATH_MAX, "http://%s%s", server, abs_path);
-	  slash_pos[1] = tmp_char;
-	  board = ochusha_bbs_table_lookup_board_by_url(table, board_url);
-
-	  if (board != NULL)
-	    {
-	      parse_thread_path(slash_pos + 1, &thread_id, &from, &to);
-	    }
-	  else
-	    board_url[0] = '\0';
-	}
-#if 0
-      fprintf(stderr, "  board_url: %s\n", board_url);
-#endif
-    }
-
-  if (board != NULL)
-    bbs_type = board->bbs_type;
-
- done:
-  if (server != NULL)
-    G_FREE(server);
-
-  if (abs_path != NULL)
-    G_FREE(abs_path);
-
-  if (board_p != NULL)
-    *board_p = board;
 
   if (board_url_p != NULL)
-    {
-      if (board_url[0] != '\0')
-	*board_url_p = G_STRDUP(board_url);
-      else
-	*board_url_p = NULL;
-    }
+    *board_url_p = board_url;
+  else if (board_url != NULL)
+    G_FREE(board_url);
 
-  if (bbs_type_p != NULL)
-    *bbs_type_p = bbs_type;
-
-  if (thread_id_p != NULL)
-    *thread_id_p = thread_id;
-  else if (thread_id != NULL)
-    G_FREE(thread_id);
-
-  if (from_p != NULL)
-    *from_p = from;
-
-  if (to_p != NULL)
-    *to_p = to;
-
-  return board_url[0] != '\0';
+  return result;
 }
 
 
