@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_bbs_thread.c,v 1.13 2003/11/18 21:02:05 fuyu Exp $
+ * $Id: ochusha_bbs_thread.c,v 1.14 2003/11/25 18:20:44 fuyu Exp $
  */
 
 #include "config.h"
@@ -493,19 +493,20 @@ ochusha_bbs_thread_is_post_supported(OchushaBBSThread *thread)
 
 gboolean
 ochusha_bbs_thread_post_response(OchushaBBSThread *thread,
-				 OchushaConfig *config,
+				 OchushaNetworkBroker *broker,
 				 const OchushaBBSResponse *response)
 {
   OchushaBBSThreadClass *klass;
 
   g_return_val_if_fail(OCHUSHA_IS_BBS_THREAD(thread)
-		       && config != NULL && response != NULL, FALSE);
+		       && OCHUSHA_IS_NETWORK_BROKER(broker)
+		       && response != NULL, FALSE);
 
   klass = OCHUSHA_BBS_THREAD_GET_CLASS(thread);
 
   g_return_val_if_fail(klass->post_response != NULL, FALSE);
 
-  return (*klass->post_response)(thread, config, response);
+  return (*klass->post_response)(thread, broker, response);
 }
 
 
