@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_bbs_thread.h,v 1.13 2003/11/23 21:13:44 fuyu Exp $
+ * $Id: ochusha_bbs_thread.h,v 1.14 2003/11/25 18:20:44 fuyu Exp $
  */
 
 #ifndef _OCHUSHA_BBS_THREAD_H_
@@ -121,7 +121,8 @@ struct _OchushaBBSThreadClass
 			       EndThreadCallback *end_cb,
 			       gpointer callback_data);
   gboolean (*post_supported)(OchushaBBSThread *thread);
-  gboolean (*post_response)(OchushaBBSThread *thread, OchushaConfig *config,
+  gboolean (*post_response)(OchushaBBSThread *thread,
+			    OchushaNetworkBroker *broker,
 			    const OchushaBBSResponse *response);
 };
 
@@ -193,7 +194,7 @@ gboolean ochusha_bbs_thread_preview_response(OchushaBBSThread *thread,
 					gpointer callback_data);
 
 gboolean ochusha_bbs_thread_post_response(OchushaBBSThread *thread,
-					  OchushaConfig *config,
+					  OchushaNetworkBroker *broker,
 					  const OchushaBBSResponse *response);
 
 #ifdef __cplusplus
