@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_board_2ch.c,v 1.19 2003/11/23 22:39:35 fuyu Exp $
+ * $Id: ochusha_board_2ch.c,v 1.20 2003/11/25 18:20:44 fuyu Exp $
  */
 
 #include "config.h"
@@ -118,7 +118,7 @@ static gboolean ochusha_board_2ch_is_new_thread_supported(
 						OchushaBulletinBoard *board);
 static gboolean ochusha_board_2ch_create_new_thread(
 					OchushaBulletinBoard *board,
-					OchushaConfig *config,
+					OchushaNetworkBroker *broker,
 					const gchar *title,
 					const OchushaBBSResponse *response);
 
@@ -511,7 +511,7 @@ ochusha_board_2ch_is_new_thread_supported(OchushaBulletinBoard *board)
 
 static gboolean
 ochusha_board_2ch_create_new_thread(OchushaBulletinBoard *board,
-				    OchushaConfig *config,
+				    OchushaNetworkBroker *broker,
 				    const gchar *title_utf8,
 				    const OchushaBBSResponse *response)
 {
@@ -604,11 +604,11 @@ ochusha_board_2ch_create_new_thread(OchushaBulletinBoard *board,
       goto error_exit;
     }
 
-  post_result = ochusha_utils_2ch_try_post(board, query);
+  post_result = ochusha_utils_2ch_try_post(broker, board, query);
   if (post_result == OCHUSHA_UTILS_2CH_POST_NO_COOKIE)
     {
       sleep(10);
-      post_result = ochusha_utils_2ch_try_post(board, query);
+      post_result = ochusha_utils_2ch_try_post(broker, board, query);
     }
 
 #if 0
@@ -616,7 +616,7 @@ ochusha_board_2ch_create_new_thread(OchushaBulletinBoard *board,
     {
       /* XXX: äݤΤȤͭʤΤ֡ */
       sleep(30);
-      post_result = ochusha_utils_2ch_try_post(thread, query);
+      post_result = ochusha_utils_2ch_try_post(broker, board, query);
     }
 #endif
   if (post_result != OCHUSHA_UTILS_2CH_POST_SUCCESS)
@@ -1049,6 +1049,7 @@ refresh_threadlist_by_subject_txt(OchushaBulletinBoard *board,
 	      {
 		thread_2ch->alive = TRUE;
 
+		thread->flags &= ~OCHUSHA_BBS_THREAD_DAT_DROPPED;
 		thread->number_of_responses_on_server = n_responses_on_server;
 
 		g_object_ref(G_OBJECT(thread));
@@ -1271,6 +1272,7 @@ refresh_threadlist_by_subback_html(OchushaBulletinBoard *board,
 	      {
 		thread_2ch->alive = TRUE;
 
+		thread->flags &= ~OCHUSHA_BBS_THREAD_DAT_DROPPED;
 		thread->number_of_responses_on_server = n_responses_on_server;
 
 		g_object_ref(G_OBJECT(thread));
