@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_bulletin_board.c,v 1.16 2003/11/23 21:13:44 fuyu Exp $
+ * $Id: ochusha_bulletin_board.c,v 1.17 2003/11/25 18:20:44 fuyu Exp $
  */
 
 #include "config.h"
@@ -654,21 +654,22 @@ ochusha_bulletin_board_is_new_thread_supported(OchushaBulletinBoard *board)
 
 gboolean
 ochusha_bulletin_board_create_new_thread(OchushaBulletinBoard *board,
-					 OchushaConfig *config,
+					 OchushaNetworkBroker *broker,
 					 const gchar *title,
 					 const OchushaBBSResponse *response)
 {
   OchushaBulletinBoardClass *klass;
 
   g_return_val_if_fail(OCHUSHA_IS_BULLETIN_BOARD(board)
-		       && config != NULL && title != NULL && response != NULL,
+		       && OCHUSHA_IS_NETWORK_BROKER(broker)
+		       && title != NULL && response != NULL,
 		       FALSE);
 
   klass = OCHUSHA_BULLETIN_BOARD_GET_CLASS(board);
 
   g_return_val_if_fail(klass->create_new_thread != NULL, FALSE);
 
-  return (*klass->create_new_thread)(board, config, title, response);
+  return (*klass->create_new_thread)(board, broker, title, response);
 }
 
 
