@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_config.c,v 1.11 2003/11/19 21:19:28 fuyu Exp $
+ * $Id: ochusha_config.c,v 1.13 2003/11/26 06:02:14 fuyu Exp $
  */
 
 #include "config.h"
@@ -663,11 +663,18 @@ getEntityHandler(void *context, const xmlChar *name)
 	  "      <boolean val=\"%s\"/>\n"			\
 	  "    </attribute>\n", (config)->attribute ? "true" : "false" )
 
-#define OUTPUT_CONFIG_ATTRIBUTE_STRING(file, config, attribute)	\
-  fprintf(file,							\
-	  "    <attribute name=\"" #attribute	"\">\n"		\
-	  "      <string>%s</string>\n"				\
-	  "    </attribute>\n", (config)->attribute)
+#define OUTPUT_CONFIG_ATTRIBUTE_STRING(file, config, attribute)		\
+  do {									\
+    if ((config)->attribute != NULL)					\
+      {									\
+	gchar *text = g_markup_escape_text((config)->attribute, -1);	\
+	fprintf(file,							\
+		"    <attribute name=\"" #attribute	"\">\n"		\
+		"      <string>%s</string>\n"				\
+		"    </attribute>\n", text);				\
+	g_free(text);							\
+      }									\
+  } while (0)
 
 
 gboolean
@@ -710,6 +717,8 @@ ochusha_write_config_xml(OchushaConfig *config,
   OUTPUT_CONFIG_ATTRIBUTE_BOOLEAN(config_xml, config, offline);
 
   OUTPUT_CONFIG_ATTRIBUTE_BOOLEAN(config_xml, config, enable_proxy);
+  OUTPUT_CONFIG_ATTRIBUTE_BOOLEAN(config_xml, config,
+				  enable_proxy_only_for_posting);
 
   if (config->proxy_url == NULL)
     config->proxy_url = empty_string;
@@ -761,6 +770,7 @@ ochusha_read_config_xml(OchushaConfig *config,
       config->offline = FALSE;
 
       config->enable_proxy = FALSE;
+      config->enable_proxy_only_for_posting = FALSE;
       config->enable_proxy_auth = FALSE;
 
       config->proxy_url = G_STRDUP("");
@@ -804,6 +814,12 @@ ochusha_read_config_xml(OchushaConfig *config,
       config->enable_proxy
 	= ochusha_utils_get_attribute_boolean(pref, "enable_proxy");
 
+      config->enable_proxy_only_for_posting
+	= ochusha_utils_get_attribute_boolean(pref,
+					      "enable_proxy_only_for_posting");
+      if (config->enable_proxy_only_for_posting)
+	config->enable_proxy = FALSE;
+
       config->proxy_url
 	= ochusha_utils_get_attribute_string(pref, "proxy_url");
       if (config->proxy_url == NULL)
