@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_network_broker.c,v 1.11 2003/11/21 21:30:03 fuyu Exp $
+ * $Id: ochusha_network_broker.c,v 1.14 2003/11/26 06:02:14 fuyu Exp $
  */
 
 #include "config.h"
@@ -63,6 +63,7 @@
 
 #define CACHE_COMPARE_SIZE	128	/* Ĥѹǽˤ٤ */
 #define HEADER_BUFFER_LENGTH	256
+#define LOG_MESSAGE_SIZE	4096
 
 #define POLLING_INTERVAL_SEC	0
 #define POLLING_INTERVAL_NANO	20000000	/* 20ms */
@@ -83,6 +84,8 @@ static struct timespec polling_interval =
 static void ochusha_network_broker_class_init(OchushaNetworkBrokerClass *klass);
 static void ochusha_network_broker_init(OchushaNetworkBroker *broker);
 static void ochusha_network_broker_finalize(GObject *object);
+static void ochusha_network_broker_output_log(OchushaNetworkBroker *broker,
+					      const gchar *message);
 
 
 GType
@@ -120,12 +123,13 @@ enum {
   ACCESS_COMPLETED_SIGNAL,
   ACCESS_TERMINATED_SIGNAL,
   ACCESS_FAILED_SIGNAL,
+  OUTPUT_LOG_SIGNAL,
   LAST_SIGNAL
 };
 
 
 static GObjectClass *parent_class = NULL;
-static gint broker_signals[LAST_SIGNAL] = { 0, 0, 0, 0, 0 };
+static gint broker_signals[LAST_SIGNAL] = { 0, 0, 0, 0, 0, 0 };
 static GQuark broker_buffer_status_id;
 static GQuark broker_job_args_id;
 
@@ -194,6 +198,15 @@ ochusha_network_broker_class_init(OchushaNetworkBrokerClass *klass)
 		 OCHUSHA_TYPE_ASYNC_BUFFER,
 		 G_TYPE_INT,
 		 G_TYPE_STRING);
+  broker_signals[OUTPUT_LOG_SIGNAL] =
+    g_signal_new("output_log",
+		 G_TYPE_FROM_CLASS(klass),
+		 G_SIGNAL_RUN_LAST,
+		 G_STRUCT_OFFSET(OchushaNetworkBrokerClass, output_log),
+		 NULL, NULL,
+		 libochusha_marshal_VOID__STRING,
+		 G_TYPE_NONE, 1,
+		 G_TYPE_STRING);
 
   broker_buffer_status_id
     = g_quark_from_static_string("OchushaNetworkBroker::BufferStatus");
@@ -205,6 +218,7 @@ ochusha_network_broker_class_init(OchushaNetworkBrokerClass *klass)
   klass->access_completed = NULL;
   klass->access_terminated = NULL;
   klass->access_failed = NULL;
+  klass->output_log = NULL;
 }
 
 
@@ -234,6 +248,17 @@ ochusha_network_broker_new(OchushaConfig *config)
 }
 
 
+static void
+ochusha_network_broker_output_log(OchushaNetworkBroker *broker,
+				  const gchar *message)
+{
+  g_signal_emit(G_OBJECT(broker),
+		broker_signals[OUTPUT_LOG_SIGNAL],
+		0,
+		message);
+}
+
+
 static void
 munmap_when_finished(OchushaAsyncBuffer *buffer)
 {
@@ -343,10 +368,11 @@ read_cache_to_buffer(const OchushaConfig *config, const char *url,
 
 
 static void
-write_buffer_to_cache(const OchushaConfig *config, const char *url,
+write_buffer_to_cache(OchushaNetworkBroker *broker, const char *url,
 		      OchushaAsyncBuffer *buffer)
 {
-  int fd = ochusha_config_cache_open_file(config, url,
+  gchar message[LOG_MESSAGE_SIZE];
+  int fd = ochusha_config_cache_open_file(broker->config, url,
 					  O_WRONLY | O_TRUNC | O_CREAT);
   if (fd >= 0)
     {
@@ -354,18 +380,28 @@ write_buffer_to_cache(const OchushaConfig *config, const char *url,
       close(fd);
       if (len != buffer->length)
 	{
-	  ochusha_config_cache_unlink_file(config, url);
+	  ochusha_config_cache_unlink_file(broker->config, url);
+	  snprintf(message, LOG_MESSAGE_SIZE,
+		   _("Couldn't update cache file for %s: %s (%d)\n"),
+		   url, strerror(errno), errno);
+	  ochusha_network_broker_output_log(broker, message);
 #if DEBUG_NETWORK
 	  fprintf(stderr, "Couldn't update cache file for %s: %s (%d)\n",
 		  url, strerror(errno), errno);
 #endif
 	}
     }
-#if DEBUG_NETWORK
   else
-    fprintf(stderr, "Couldn't open cache file for %s: %s (%d)\n",
-	    url, strerror(errno), errno);
+    {
+      snprintf(message, LOG_MESSAGE_SIZE,
+	       _("Couldn't open cache file for %s: %s (%d)\n"),
+	       url, strerror(errno), errno);
+      ochusha_network_broker_output_log(broker, message);
+#if DEBUG_NETWORK
+      fprintf(stderr, "Couldn't open cache file for %s: %s (%d)\n",
+	      url, strerror(errno), errno);
 #endif
+    }
 }
 
 
@@ -375,12 +411,17 @@ write_buffer_to_cache(const OchushaConfig *config, const char *url,
 
 
 static gboolean
-setup_common_request_headers(const OchushaConfig *config,
-			     ghttp_request *request)
+setup_common_request_headers(OchushaNetworkBroker *broker,
+			     ghttp_request *request,
+			     gboolean posting)
 {
+  gchar message[LOG_MESSAGE_SIZE];
   gboolean result = TRUE;
+  const OchushaConfig *config = broker->config;
 
-  if (config->enable_proxy && config->proxy_url != NULL)
+  if ((config->enable_proxy
+       || (config->enable_proxy_only_for_posting && posting))
+      && config->proxy_url != NULL)
     {
       int result = ghttp_set_proxy(request, config->proxy_url);
       if (result == 0)
@@ -392,14 +433,26 @@ setup_common_request_headers(const OchushaConfig *config,
 						config->proxy_password);
 	      if (result != 0)
 		{
+		  snprintf(message, LOG_MESSAGE_SIZE,
+			   _("Invalid proxy auth info: user=\"%s\", password=\"%s\"\n"),
+			   config->proxy_user, config->proxy_password);
+		  ochusha_network_broker_output_log(broker, message);
+#if DEBUG_NETWORK_MOST
 		  fprintf(stderr, "Invalid proxy auth info.\n");
+#endif
 		  result = FALSE;
 		}
 	    }
 	}
       else
 	{
+	  snprintf(message, LOG_MESSAGE_SIZE,
+		   _("Invalid proxy URL: \"%s\"\n"),
+		   config->proxy_url);
+	  ochusha_network_broker_output_log(broker, message);
+#if DEBUG_NETWORK_MOST
 	  fprintf(stderr, "Invalid proxy URL: %s\n", config->proxy_url);
+#endif
 	  result = FALSE;
 	}
     }
@@ -469,6 +522,7 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
   const char *error_message = NULL;
   const char *connection;
   gboolean connection_closed = FALSE;
+  gchar message[LOG_MESSAGE_SIZE];
 
   /* ʲϸľɬ */
 #if DEBUG_NETWORK
@@ -515,7 +569,7 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
   else
     if_modified_since = NULL;
 
-  if (!setup_common_request_headers(broker->config, request))
+  if (!setup_common_request_headers(broker, request, FALSE))
     {
       g_signal_emit(G_OBJECT(broker),
 		    broker_signals[ACCESS_FAILED_SIGNAL],
@@ -536,6 +590,9 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
 		buffer,
 		&signal_result);
 
+  snprintf(message, LOG_MESSAGE_SIZE, _("Starting HEAD request: %s\n"), url);
+  ochusha_network_broker_output_log(broker, message);
+
   ghttp_set_sync(request, ghttp_async);
   ghttp_prepare(request);
 
@@ -599,8 +656,14 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
 
 #if DEBUG_NETWORK
       if (status_code != 0)
-	fprintf(stderr, "status_code=%d (%s)\n",
-		status_code, ghttp_reason_phrase(request));
+	{
+	  snprintf(message, LOG_MESSAGE_SIZE,
+		   _("Result of HEAD request(%s): status_code=%d (%s)\n"),
+		   url, status_code, ghttp_reason_phrase(request));
+	  ochusha_network_broker_output_log(broker, message);
+	  fprintf(stderr, "status_code=%d (%s)\n",
+		  status_code, ghttp_reason_phrase(request));
+	}
       else
 	fprintf(stderr, "Unknown error.\n");
 #endif
@@ -627,6 +690,10 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
     }
 
 # if DEBUG_NETWORK_MOST
+  snprintf(message, LOG_MESSAGE_SIZE,
+	   _("Result of HEAD request(%s): status_code=%d (%s)\n"),
+	   url, status_code, ghttp_reason_phrase(request));
+  ochusha_network_broker_output_log(broker, message);
   fprintf(stderr, "status_code=%d (%s)\n",
 	  status_code, ghttp_reason_phrase(request));
 # endif
@@ -650,7 +717,7 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
   ghttp_set_type(request, ghttp_type_get);
   ghttp_set_header(request, http_hdr_Connection, "close");
   ghttp_set_header(request, http_hdr_Accept_Encoding, "deflate, gzip");
-  if (!setup_common_request_headers(broker->config, request))
+  if (!setup_common_request_headers(broker, request, FALSE))
     {
       g_signal_emit(G_OBJECT(broker),
 		    broker_signals[ACCESS_FAILED_SIGNAL],
@@ -665,6 +732,9 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
       return FALSE;
     }    
 
+  snprintf(message, LOG_MESSAGE_SIZE, _("Starting GET request: %s\n"), url);
+  ochusha_network_broker_output_log(broker, message);
+
   ghttp_set_sync(request, ghttp_async);
   ghttp_prepare(request);
 
@@ -682,14 +752,20 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
 	ghttp_flush_response_buffer(request);
       else if (state == ghttp_error)
 	{
+#if 1
+	  error_message = _("Unknown I/O error.");
+	  goto error_exit;
+#else
 	  /* XXX:  */
 	  status_code = ghttp_status_code(request);
+
 	  if (status_code != 200)
 	    {
 	      error_message = _("Unknown I/O error.");
 	      goto error_exit;
 	    }
 	  state = ghttp_done;
+#endif
 	}
 
       body = ghttp_get_body(request);
@@ -757,10 +833,16 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
 		{
 		  gzip_buffer_free(gzip_buffer);
 		  gzip_buffer = NULL;
-#if DEBUG_NETWORK
 		  if (state != ghttp_done)
-		    fprintf(stderr, "Requested file contains garbage?\n");
+		    {
+		      snprintf(message, LOG_MESSAGE_SIZE,
+			       _("Requested file seems to contain garbage: %s\n"),
+			       url);
+		      ochusha_network_broker_output_log(broker, message);
+#if DEBUG_NETWORK
+		      fprintf(stderr, "Requested file seems to contain garbage: %s\n", url);
 #endif
+		    }
 		  break;
 		}
 	    }
@@ -778,6 +860,11 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
 	      status->state
 		= OCHUSHA_NETWORK_BROKER_BUFFER_STATE_ACCESS_FAILED;
 
+	      snprintf(message, LOG_MESSAGE_SIZE,
+		       _("Access terminated: %s\n"),
+		       url);
+	      ochusha_network_broker_output_log(broker, message);
+
 	      return FALSE;
 	    }
 	}
@@ -822,6 +909,11 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
       ghttp_request_destroy(request);
       args->request = NULL;
 
+      snprintf(message, LOG_MESSAGE_SIZE,
+	       _("Access completed: %s\n"),
+	       url);
+      ochusha_network_broker_output_log(broker, message);
+
       return TRUE;
     }
 
@@ -829,21 +921,9 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
   status_code = ghttp_status_code(request);
   if (error_message != NULL)
     {
-      g_signal_emit(G_OBJECT(broker),
-		    broker_signals[ACCESS_FAILED_SIGNAL],
-		    0,
-		    buffer,
-		    OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
-		    error_message,
-		    &signal_result);
-      fprintf(stderr, "access failed: status_code=%d %s\n",
-	      status_code, error_message);
-    }
-  else if (status_code != 0)
-    {
-      char message[4096];
-      snprintf(message, 4096, "Access failed: %d (%s)",
-	       status_code, ghttp_reason_phrase(request));
+      snprintf(message, LOG_MESSAGE_SIZE,
+	       _("Access failed(%s): status_code=%d (%s)\n"),
+	       url, status_code, error_message);
       g_signal_emit(G_OBJECT(broker),
 		    broker_signals[ACCESS_FAILED_SIGNAL],
 		    0,
@@ -851,16 +931,46 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
 		    OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
 		    message,
 		    &signal_result);
+      ochusha_network_broker_output_log(broker, message);
+#if DEBUG_NETWORK_MOST
+      fprintf(stderr, "access failed: status_code=%d %s\n",
+	      status_code, error_message);
+#endif
+    }
+  else if (status_code != 0)
+    {
+      snprintf(message, LOG_MESSAGE_SIZE, _("Access failed(%s): %d (%s)"),
+	       url, status_code, ghttp_reason_phrase(request));
+      g_signal_emit(G_OBJECT(broker),
+		    broker_signals[ACCESS_FAILED_SIGNAL],
+		    0,
+		    buffer,
+		    OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
+		    message,
+		    &signal_result);
+      ochusha_network_broker_output_log(broker, message);
+#if DEBUG_NETWORK_MOST
       fprintf(stderr, "%s\n", message);
+#endif
     }
   else
-    g_signal_emit(G_OBJECT(broker),
-		  broker_signals[ACCESS_FAILED_SIGNAL],
-		  0,
-		  buffer,
-		  OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
-		  _("Access failed: unknown reason."),
-		  &signal_result);
+    {
+      snprintf(message, LOG_MESSAGE_SIZE,
+	       _("Access failed(%s): unknown reason.\n"),
+	       url);
+      g_signal_emit(G_OBJECT(broker),
+		    broker_signals[ACCESS_FAILED_SIGNAL],
+		    0,
+		    buffer,
+		    OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
+		    message,
+		    &signal_result);
+      ochusha_network_broker_output_log(broker, message);
+#if DEBUG_NETWORK_MOST
+      fprintf(stderr, "%s\n", message);
+#endif
+    }
+
   ghttp_request_destroy(request);
   args->request = NULL;
 
@@ -966,6 +1076,7 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
   ContentEncodingType content_encoding = UNKNOWN_ENCODING;
   int status_code;
   const char *error_message = NULL;
+  gchar message[LOG_MESSAGE_SIZE];
 
   if (!ochusha_async_buffer_active_ref(buffer,
 			"ochusha_network_broker.c: try_update_cache()"))
@@ -990,6 +1101,10 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
       g_object_set_qdata(G_OBJECT(buffer), broker_job_args_id, NULL);
       g_object_unref(G_OBJECT(buffer));
 
+      snprintf(message, LOG_MESSAGE_SIZE,
+	       _("Access terminated: %s\n"), url);
+      ochusha_network_broker_output_log(args->broker, message);
+
       return;
     }
 
@@ -1038,7 +1153,7 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 	  fprintf(stderr, "If-Modified-Since: %s\n", args->if_modified_since);
 #endif
 	}
-      if (!setup_common_request_headers(args->broker->config, request))
+      if (!setup_common_request_headers(args->broker, request, FALSE))
 	{
 	  /* ۤäƥåϤɤΤ */
 	  status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_CACHE_IS_OK;
@@ -1061,6 +1176,10 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 		    buffer,
 		    &signal_result);
 
+      snprintf(message, LOG_MESSAGE_SIZE,
+	       _("Starting HEAD request: %s\n"), url);
+      ochusha_network_broker_output_log(args->broker, message);
+
       ghttp_set_sync(request, ghttp_async);
       ghttp_prepare(request);
 
@@ -1088,6 +1207,11 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 	      ghttp_request_destroy(request);
 	      args->request = NULL;
 
+	      snprintf(message, LOG_MESSAGE_SIZE,
+		       _("Access terminated: %s\n"),
+		       url);
+	      ochusha_network_broker_output_log(args->broker, message);
+
 	      goto finish_try_update_cache;
 	    }
 	  if (state != ghttp_not_done)
@@ -1120,6 +1244,10 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 	  ghttp_request_destroy(request);
 	  args->request = NULL;
 
+	  snprintf(message, LOG_MESSAGE_SIZE,
+		   _("Cache is fresh: %s\n"), url);
+	  ochusha_network_broker_output_log(args->broker, message);
+
 	  goto finish_try_update_cache;
 	}
 
@@ -1188,7 +1316,7 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 #endif
 	  ghttp_set_header(request, http_hdr_Range, header);
 
-	  if (!setup_common_request_headers(args->broker->config, request))
+	  if (!setup_common_request_headers(args->broker, request, FALSE))
 	    {
 	      status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_CACHE_IS_OK;
 	      g_signal_emit(G_OBJECT(args->broker),
@@ -1248,6 +1376,10 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 	      ghttp_request_destroy(request);
 	      args->request = NULL;
 
+	      snprintf(message, LOG_MESSAGE_SIZE,
+		       _("Access terminated: %s\n"), url);
+	      ochusha_network_broker_output_log(args->broker, message);
+
 	      goto finish_try_update_cache;
 	    }
 
@@ -1373,6 +1505,10 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 		      ghttp_request_destroy(request);
 		      args->request = NULL;
 
+		      snprintf(message, LOG_MESSAGE_SIZE,
+			       _("Access terminated: %s\n"), url);
+		      ochusha_network_broker_output_log(args->broker, message);
+
 		      goto finish_try_update_cache;
 		    }
 
@@ -1429,7 +1565,12 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 			    &signal_result);
 	      ghttp_request_destroy(request);
 	      args->request = NULL;
-	      write_buffer_to_cache(args->broker->config, url, buffer);
+	      write_buffer_to_cache(args->broker, url, buffer);
+
+	      snprintf(message, LOG_MESSAGE_SIZE,
+		       _("Cache updated: %s\n"), url);
+	      ochusha_network_broker_output_log(args->broker, message);
+
 	      goto finish_try_update_cache;
 	    }
 	  /*
@@ -1463,6 +1604,10 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 	      ghttp_request_destroy(request);
 	      args->request = NULL;
 
+	      snprintf(message, LOG_MESSAGE_SIZE,
+		       _("Cache is fresh: %s\n"), url);
+	      ochusha_network_broker_output_log(args->broker, message);
+
 	      goto finish_try_update_cache;
 	    }
 
@@ -1486,6 +1631,10 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
       ghttp_request_destroy(request);
       args->request = NULL;
 
+      snprintf(message, LOG_MESSAGE_SIZE,
+	       _("Cache is dirty: %s\n"), url);
+      ochusha_network_broker_output_log(args->broker, message);
+
       if (!ochusha_async_buffer_update_length(buffer, 0,
 			"ochusha_network_broker.c: try_update_cache()"))
 	{
@@ -1504,6 +1653,10 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 		_("Access terminated."),
 		&signal_result);
 
+	  snprintf(message, LOG_MESSAGE_SIZE,
+		   _("Access terminated: %s\n"), url);
+	  ochusha_network_broker_output_log(args->broker, message);
+
 	  goto finish_try_update_cache;
 	}
     }
@@ -1520,7 +1673,7 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
       if (status->state == OCHUSHA_NETWORK_BROKER_BUFFER_STATE_DIRECT_ACCESS)
 	status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_ACCESS_OK;
 
-      write_buffer_to_cache(args->broker->config, url, buffer);
+      write_buffer_to_cache(args->broker, url, buffer);
     }
   /* 褿硢顼äȤƤ⥷ʥȯ */
   goto finish_try_update_cache;
@@ -1528,6 +1681,8 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
  error_exit:
   if (error_message != NULL)
     {
+      snprintf(message, LOG_MESSAGE_SIZE,
+	       _("Access failed(%s): %s\n"), url, error_message);
       g_signal_emit(G_OBJECT(args->broker),
 		    broker_signals[ACCESS_FAILED_SIGNAL],
 		    0,
@@ -1535,15 +1690,15 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 		    OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
 		    error_message,
 		    &signal_result);
+      ochusha_network_broker_output_log(args->broker, message);
     }
   else
     {
       status_code = ghttp_status_code(request);
       if (status_code != 0)
 	{
-	  char message[4096];
-	  snprintf(message, 4096, "Access failed: %d (%s)",
-		   status_code, ghttp_reason_phrase(request));
+	  snprintf(message, LOG_MESSAGE_SIZE, _("Access failed(%s): %d (%s)"),
+		   url, status_code, ghttp_reason_phrase(request));
 	  g_signal_emit(G_OBJECT(args->broker),
 			broker_signals[ACCESS_FAILED_SIGNAL],
 			0,
@@ -1551,15 +1706,22 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 			OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
 			message,
 			&signal_result);
+	  ochusha_network_broker_output_log(args->broker, message);
 	}
       else
-	g_signal_emit(G_OBJECT(args->broker),
-		      broker_signals[ACCESS_FAILED_SIGNAL],
-		      0,
-		      buffer,
-		      OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
-		      _("Access failed: unknown reason."),
-		      &signal_result);
+	{
+	  snprintf(message, LOG_MESSAGE_SIZE,
+		   _("Access failed(%s): unknown reason.\n"),
+		   url);
+	  g_signal_emit(G_OBJECT(args->broker),
+			broker_signals[ACCESS_FAILED_SIGNAL],
+			0,
+			buffer,
+			OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
+			message,
+			&signal_result);
+	  ochusha_network_broker_output_log(args->broker, message);
+	}
     }
 
   if (request != NULL)
@@ -1585,6 +1747,7 @@ refresh_cache_after_read(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 					      broker_job_args_id);
   OchushaNetworkBrokerBufferStatus *status
     = g_object_get_qdata(G_OBJECT(buffer), broker_buffer_status_id);
+  gchar message[LOG_MESSAGE_SIZE];
 
 #if DEBUG_THREAD_MOST
   fprintf(stderr, "refresh_cache_after_read() thread is invoked.\n");
@@ -1609,7 +1772,7 @@ refresh_cache_after_read(WorkerThread *employee, OchushaAsyncBuffer *buffer)
       /* ̿ */
       status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_ACCESS_OK;
       /* ХåեƤ򥭥å˽񤭹ࡣ*/
-      write_buffer_to_cache(args->broker->config, args->url, buffer);
+      write_buffer_to_cache(args->broker, args->url, buffer);
     }
   else
     {
@@ -1636,6 +1799,10 @@ refresh_cache_after_read(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 			    OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
 			    _("Couldn't read from network, cache used."),
 			    &signal_result);
+	      snprintf(message, LOG_MESSAGE_SIZE,
+		       _("Use cached file(%s): due to networking error.\n"),
+		       args->url);
+	      ochusha_network_broker_output_log(args->broker, message);
 	    }
 	}
       else
@@ -1650,6 +1817,10 @@ refresh_cache_after_read(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 			OCHUSHA_NETWORK_BROKER_FAILURE_REASON_CACHE_NOT_FOUND,
 			_("Couldn't find cache that should exist."),
 			&signal_result);
+	      snprintf(message, LOG_MESSAGE_SIZE,
+		       _("Couldn't find cache file: %s\n"),
+		       args->url);
+	      ochusha_network_broker_output_log(args->broker, message);
 	    }
 	  else
 	    {
@@ -1904,7 +2075,7 @@ http_read_from_url_synchronously(OchushaNetworkBroker *broker,
   ghttp_set_header(request, http_hdr_Accept_Encoding, "deflate, gzip");
   ghttp_set_header(request, http_hdr_Connection, "close");
 
-  if (!setup_common_request_headers(broker->config, request))
+  if (!setup_common_request_headers(broker, request, FALSE))
     {
       g_signal_emit(G_OBJECT(broker),
 		    broker_signals[ACCESS_FAILED_SIGNAL],
@@ -2082,8 +2253,8 @@ http_read_from_url_synchronously(OchushaNetworkBroker *broker,
     }
   else if (status_code != 0)
     {
-      char message[4096];
-      snprintf(message, 4096, "Access failed: %d (%s)",
+      char message[LOG_MESSAGE_SIZE];
+      snprintf(message, LOG_MESSAGE_SIZE, "Access failed: %d (%s)",
 	       status_code, ghttp_reason_phrase(request));
       g_signal_emit(G_OBJECT(broker),
 		    broker_signals[ACCESS_FAILED_SIGNAL],
@@ -2254,3 +2425,82 @@ ochusha_network_broker_get_header_names(OchushaNetworkBroker *broker,
 
   return ghttp_get_header_names(args->request, headers, num_headers);
 }
+
+
+#if 0
+typedef struct _OchushaNetworkBrokerPostStatus
+{
+  int status_code;
+  char *body;
+  char *set_cookie;
+} OchushaNetworkBrokerPostStatus;
+#endif
+
+gboolean
+ochusha_network_broker_try_post(OchushaNetworkBroker *broker,
+				char *url, const char *server,
+				const char *referer,
+				const char *cookie, const char *body,
+				OchushaNetworkBrokerPostStatus *status)
+{
+  ghttp_request *request = ghttp_request_new();
+
+  g_return_val_if_fail(request != NULL, FALSE);
+
+  ghttp_set_uri(request, url);
+  ghttp_set_type(request, ghttp_type_post);
+
+  setup_common_request_headers(broker, request, TRUE);
+  ghttp_set_header(request, http_hdr_Host, server);
+
+  /* ɬפʤΤ */
+#if 0
+  ghttp_set_header(request, http_hdr_Accept, "text/xml,application/xml,application/xhtml+xml,text/html;q=0.9,text/plain;q=0.8,video/x-mng,image/png,image/jpeg,image/gif;q=0.2,text/css,*/*;q=0.1");
+  ghttp_set_header(request, http_hdr_Accept_Language, "ja,en-us;q=0.66,en;q=0.33");
+  ghttp_set_header(request, http_hdr_Accept_Charset, "EUC-JP,utf-8;q=0.66,*;q=0.66");
+#endif
+
+  ghttp_set_header(request, http_hdr_Connection, "close");
+  ghttp_set_header(request, http_hdr_Referrer, referer);
+  ghttp_set_header(request, "Referer", referer);
+
+  if (cookie != NULL)
+    ghttp_set_header(request, "Cookie", cookie);
+
+  ghttp_set_body(request, (char *)body, strlen(body));
+  ghttp_prepare(request);
+
+  if (ghttp_process(request) == ghttp_error)
+    {
+#if DEBUG_POST
+      fprintf(stderr, "ghttp_process() returns ghttp_error\n");
+#endif
+      ghttp_request_destroy(request);
+      if (status != NULL)
+	{
+	  status->status_code = 0;
+	  status->body = NULL;
+	  status->set_cookie = NULL;
+	}
+
+      return FALSE;
+    }
+
+  if (status != NULL)
+    {
+      const char *set_cookie;
+      status->status_code = ghttp_status_code(request);
+      status->body = G_MEMDUP(ghttp_get_body(request),
+			      ghttp_get_body_len(request));
+      set_cookie = ghttp_get_header(request, http_hdr_Set_Cookie);
+      if (set_cookie != NULL)
+	status->set_cookie = G_STRDUP(set_cookie);
+      else
+	status->set_cookie = NULL;
+    }
+
+  ghttp_request_destroy(request);
+
+  return TRUE;
+}
+
