@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_network_broker.h,v 1.3 2003/11/18 21:02:05 fuyu Exp $
+ * $Id: ochusha_network_broker.h,v 1.5 2003/11/25 18:20:44 fuyu Exp $
  */
 
 #ifndef _OCHUSHA_NETWORK_BROKER_H_
@@ -100,6 +100,7 @@ struct _OchushaNetworkBrokerClass
 			    OchushaAsyncBuffer *buffer,
 			    OchushaNetworkBrokerFailureReason reason_code,
 			    const gchar *reason);
+  void (*output_log)(OchushaNetworkBroker *broker, const gchar *message);
 };
 
 
@@ -144,6 +145,19 @@ int ochusha_network_broker_get_header_names(OchushaNetworkBroker *broker,
 					    OchushaAsyncBuffer *buffer,
 					    char ***headers, int *num_headers);
 
+typedef struct _OchushaNetworkBrokerPostStatus
+{
+  int status_code;
+  char *body;
+  char *set_cookie;
+} OchushaNetworkBrokerPostStatus;
+
+
+gboolean ochusha_network_broker_try_post(OchushaNetworkBroker *broker,
+				char *url, const char *server,
+				const char *referer,
+				const char *cookie, const char *body,
+				OchushaNetworkBrokerPostStatus *status);
 
 #ifdef __cplusplus
 } // extern "C"
