@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_thread_2ch.c,v 1.26 2003/11/23 21:13:44 fuyu Exp $
+ * $Id: ochusha_thread_2ch.c,v 1.28 2003/11/25 18:20:44 fuyu Exp $
  */
 
 #include "config.h"
@@ -97,7 +97,7 @@ static gboolean ochusha_thread_2ch_preview_response(OchushaBBSThread *thread,
 static gboolean ochusha_thread_2ch_post_supported(OchushaBBSThread *thread);
 
 static gboolean ochusha_thread_2ch_post_response(OchushaBBSThread *thread,
-					OchushaConfig *config,
+					OchushaNetworkBroker *broker,
 					const OchushaBBSResponse *response);
 
 static char *ochusha_thread_2ch_get_url_for_response(OchushaBBSThread *thread,
@@ -736,6 +736,7 @@ ochusha_thread_2ch_parse_responses(OchushaBBSThread *thread,
   unsigned int *responses;
   gboolean result = TRUE;
   OchushaNetworkBrokerBufferStatus *status;
+  OchushaNetworkBrokerBufferStatus fake_status;
 
   g_return_val_if_fail(OCHUSHA_IS_THREAD_2CH(thread)
 		       && OCHUSHA_IS_ASYNC_BUFFER(buffer), FALSE);
@@ -743,7 +744,14 @@ ochusha_thread_2ch_parse_responses(OchushaBBSThread *thread,
   status = g_object_get_data(G_OBJECT(buffer),
 			     "OchushaNetworkBroker::BufferStatus");
 
-  g_return_val_if_fail(status != NULL, FALSE);
+  if (status == NULL)
+    {
+      fake_status.state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_CACHE_HIT;
+      fake_status.status_code = 304;
+      fake_status.last_modified = NULL;
+      fake_status.date = NULL;
+      status = &fake_status;
+    }
 
   if (!ochusha_async_buffer_active_ref(buffer, "ochusha_thread_2ch.c: ochusha_thread_2ch_parse_responses"))
     {
@@ -1116,7 +1124,7 @@ ochusha_thread_2ch_post_supported(OchushaBBSThread *thread)
 
 static gboolean
 ochusha_thread_2ch_post_response(OchushaBBSThread *thread,
-				 OchushaConfig *config,
+				 OchushaNetworkBroker *broker,
 				 const OchushaBBSResponse *response)
 {
   OchushaThread2ch *thread_2ch;
@@ -1204,11 +1212,11 @@ ochusha_thread_2ch_post_response(OchushaBBSThread *thread,
       goto error_exit;
     }
 
-  post_result = ochusha_utils_2ch_try_post(thread->board, query);
+  post_result = ochusha_utils_2ch_try_post(broker, thread->board, query);
   if (post_result == OCHUSHA_UTILS_2CH_POST_NO_COOKIE)
     {
       sleep(10);
-      post_result = ochusha_utils_2ch_try_post(thread->board, query);
+      post_result = ochusha_utils_2ch_try_post(broker, thread->board, query);
     }
 
 #if 0
@@ -1216,7 +1224,7 @@ ochusha_thread_2ch_post_response(OchushaBBSThread *thread,
     {
       /* XXX: äݤΤȤͭʤΤ֡ */
       sleep(30);
-      post_result = ochusha_utils_2ch_try_post(thread->board, query);
+      post_result = ochusha_utils_2ch_try_post(broker, thread->board, query);
     }
 #endif
 
