@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_utils_2ch.c,v 1.16 2003/11/23 21:13:44 fuyu Exp $
+ * $Id: ochusha_utils_2ch.c,v 1.19 2003/11/25 18:20:44 fuyu Exp $
  */
 
 #include "config.h"
@@ -70,6 +70,33 @@
 #define GUESS_2CH_BOARD_URL	1
 
 
+static gboolean
+is_thread_id(const char *string)
+{
+  int len;
+  const char *pos;
+  g_return_val_if_fail(string != NULL, FALSE);
+
+  len = strlen(string);
+  if (len < 9)
+    return FALSE;
+
+  pos = string;
+  if (strchr("123456789", *pos) == NULL)
+    return FALSE;
+
+  pos++;
+  while (*pos != '\0')
+    {
+      if (strchr("0123456789", *pos) == NULL)
+	return FALSE;
+      pos++;
+    }
+
+  return TRUE;
+}
+
+
 static gboolean
 parse_thread_path(char *path, char **thread_id_p,
 		  unsigned int *from_p, unsigned int *to_p)
@@ -90,32 +117,42 @@ parse_thread_path(char *path, char **thread_id_p,
 	thread_id = G_STRNDUP(path, slash_pos - path);
       else
 	thread_id = path;
-      path = slash_pos + 1;
 
-      if (*path != '\0')
+      if (is_thread_id(thread_id))
 	{
-	  char tmp_char;
-	  char *hyphen_pos = strchr(path, '-');
-	  if (hyphen_pos != NULL)
-	    {
-	      tmp_char = *hyphen_pos;
-	      *hyphen_pos = '\0';
-	    }
-	  else
-	    tmp_char = '\0';
+	  path = slash_pos + 1;
 
-	  if (hyphen_pos != path)
+	  if (*path != '\0')
 	    {
-	      if (sscanf(path, "%d", &from) != 1)
-		from = 0;
-	    }
+	      char tmp_char;
+	      char *hyphen_pos = strchr(path, '-');
+	      if (hyphen_pos != NULL)
+		{
+		  tmp_char = *hyphen_pos;
+		  *hyphen_pos = '\0';
+		}
+	      else
+		tmp_char = '\0';
 
-	  if (hyphen_pos != NULL && hyphen_pos[1] != '\0')
-	    {
-	      if (sscanf(hyphen_pos + 1, "%d", &to) != 1)
-		to = from;
+	      if (hyphen_pos != path)
+		{
+		  if (sscanf(path, "%d", &from) != 1)
+		    from = 0;
+		}
+
+	      if (hyphen_pos != NULL && hyphen_pos[1] != '\0')
+		{
+		  if (sscanf(hyphen_pos + 1, "%d", &to) != 1)
+		    to = from;
+		}
 	    }
 	}
+      else
+	{
+	  if (thread_id != path)
+	    G_FREE(thread_id);
+	  thread_id = NULL;
+	}
     }
   else
     {
@@ -447,7 +484,7 @@ ochusha_utils_2ch_check_url(const char *url, char **board_url_p,
 	       * http://jbbs.shitaraba.com/bbs/read.cgi/computer/351/1040452877/109
 	       * URL
 	       */
-	      char *slash_pos = strchr(tmp_pos + 1, '/');
+	      char *slash_pos = strchr(tmp_pos + 13, '/');
 	      if (slash_pos != NULL)
 		{
 		  char *board_id_head = slash_pos + 1;
@@ -988,16 +1025,16 @@ ochusha_utils_2ch_analyze_bbsmenu(OchushaBBSTable *table,
 
 
 OchushaUtils2chPostResult
-ochusha_utils_2ch_try_post(OchushaBulletinBoard *board, const char *message)
+ochusha_utils_2ch_try_post(OchushaNetworkBroker *broker,
+			   OchushaBulletinBoard *board, const char *message)
 {
   char url[PATH_MAX];
-  ghttp_request *request;
   const char *cookie;
   const char *new_cookie = NULL;
-  int status_code;
   OchushaBoard2ch *board_2ch;
-  gboolean post_error = FALSE;
+  gboolean post_result = FALSE;
   int result;
+  OchushaNetworkBrokerPostStatus post_status;
 
   g_return_val_if_fail(OCHUSHA_IS_BOARD_2CH(board),
 		       OCHUSHA_UTILS_2CH_POST_FAILURE);
@@ -1029,113 +1066,65 @@ ochusha_utils_2ch_try_post(OchushaBulletinBoard *board, const char *message)
 #endif
 
   board_2ch = OCHUSHA_BOARD_2CH(board);
-
-  request = ghttp_request_new();
-
-  ghttp_set_uri(request, url);
-  ghttp_set_type(request, ghttp_type_post);
-
-  ghttp_set_header(request, http_hdr_Host,
-		   ochusha_bulletin_board_get_server(board));
-  ghttp_set_header(request, http_hdr_User_Agent, OCHUSHA_USER_AGENT);
-
-  /* ɬפʤΤ */
-  ghttp_set_header(request, http_hdr_Accept, "text/xml,application/xml,application/xhtml+xml,text/html;q=0.9,text/plain;q=0.8,video/x-mng,image/png,image/jpeg,image/gif;q=0.2,text/css,*/*;q=0.1");
-  ghttp_set_header(request, http_hdr_Accept_Language, "ja,en-us;q=0.66,en;q=0.33");
-  ghttp_set_header(request, http_hdr_Accept_Charset, "EUC-JP,utf-8;q=0.66,*;q=0.66");
-  ghttp_set_header(request, http_hdr_Connection, "close");
-  ghttp_set_header(request, http_hdr_Referrer,
-		   ochusha_bulletin_board_get_base_url(board));
-  ghttp_set_header(request, "Referer",
-		   ochusha_bulletin_board_get_base_url(board));
-
   cookie = ochusha_board_2ch_get_cookie(board_2ch);
-  if (cookie != NULL)
+  post_result = ochusha_network_broker_try_post(broker, url,
+				ochusha_bulletin_board_get_server(board),
+				ochusha_bulletin_board_get_base_url(board),
+				cookie, message, &post_status);
+
+  if (post_result)
     {
-#if DEBUG_POST_MOST
-      fprintf(stderr, "Cookie: %s\n", cookie);
-#endif
-      ghttp_set_header(request, "Cookie", cookie);
-    }
-
-  ghttp_set_body(request, (char *)message, strlen(message));
-#if DEBUG_POST_MOST
-  fprintf(stderr, "Post-Body: %s\n", message);
-#endif
-
-  ghttp_prepare(request);
-  if (ghttp_process(request) == ghttp_error)
-    {
-#if DEBUG_POST
-      fprintf(stderr, "ghttp_process() returns ghttp_error\n");
-#endif
-      ghttp_request_destroy(request);
-      ochusha_board_2ch_set_cookie(board_2ch, NULL);
-    }
-
-  status_code = ghttp_status_code(request);
-
-  if (status_code == 200)
-    {
-      const char *body = ghttp_get_body(request);
-      int len = ghttp_get_body_len(request);
-      const char *x_2ch = g_strstr_len(body, len, "2ch_X:");
-#if DEBUG_POST_MOST
-      char *tmp_body = g_strndup(body, len);
-      fprintf(stderr, "Set-Cookie: %s\n",
-	      ghttp_get_header(request, http_hdr_Set_Cookie));
-      fprintf(stderr, "Result-Body: %s\n", tmp_body);
-      g_free(tmp_body);
-#endif
-
-      if (x_2ch != NULL)
+      if (post_status.status_code == 200)
 	{
-	  if (g_strstr_len(x_2ch, len, "2ch_X:cookie") != NULL)
+	  const char *x_2ch = strstr(post_status.body, "2ch_X:");
+#if DEBUG_POST_MOST
+	  fprintf(stderr, "Set-Cookie: %s\n", post_status.set_cookie);
+	  fprintf(stderr, "Result-Body: %s\n", post_status.body);
+#endif
+	  if (x_2ch != NULL)
 	    {
-	      new_cookie = ghttp_get_header(request, http_hdr_Set_Cookie);
-	      if (new_cookie != NULL && *new_cookie != '\0')
+	      if (strstr(x_2ch, "2ch_X:cookie") != NULL)
 		{
-		  char *pos = strchr(new_cookie, ';');
-		  if (pos != NULL)
+		  new_cookie = post_status.set_cookie;
+		  if (new_cookie != NULL && *new_cookie != '\0')
 		    {
-		      char *tmp_cookie = g_strndup(new_cookie,
-						   pos - new_cookie);
-		      ochusha_board_2ch_set_cookie(board_2ch, tmp_cookie);
-		      g_free(tmp_cookie);
+		      char *pos = strchr(new_cookie, ';');
+		      if (pos != NULL)
+			{
+			  *pos = '\0';
+			  ochusha_board_2ch_set_cookie(board_2ch, new_cookie);
+			}
 		    }
 		}
-	    }
-	  else
-	    {
+	      else
+		{
 #if DEBUG_POST
-	      char *tmp_body = g_strndup(body, len);
-	      fprintf(stderr, "Set-Cookie: %s\n",
-		      ghttp_get_header(request, http_hdr_Set_Cookie));
-	      fprintf(stderr, "Result-Body: %s\n", tmp_body);
-	      g_free(tmp_body);
+		  fprintf(stderr, "Set-Cookie: %s\n", post_status.set_cookie);
+		  fprintf(stderr, "Result-Body: %s\n", post_status.body);
 #endif
-	      post_error = TRUE;
+		  post_result = FALSE;
+		}
 	    }
 	}
-    }
 #if DEBUG_POST
-  else
-    {
-      const char *body = ghttp_get_body(request);
-      int len = ghttp_get_body_len(request);
-      char *tmp_body = g_strndup(body, len);
-      fprintf(stderr, "status_code = %d\n", status_code);
-      fprintf(stderr, "Set-Cookie: %s\n",
-	      ghttp_get_header(request, http_hdr_Set_Cookie));
-      fprintf(stderr, "Result-Body: %s\n", tmp_body);
-      g_free(tmp_body);
-    }
+      else
+	{
+	  fprintf(stderr, "status_code = %d\n", post_status.status_code);
+	  fprintf(stderr, "Set-Cookie: %s\n", post_status.set_cookie);
+	  fprintf(stderr, "Result-Body: %s\n", post_status.body);
+	}
 #endif
+    }
 
-  ghttp_request_destroy(request);
+  if (post_status.body != NULL)
+    G_FREE(post_status.body);
+
+  if (post_status.set_cookie != NULL)
+    G_FREE(post_status.set_cookie);
 
   if (new_cookie != NULL)
     return OCHUSHA_UTILS_2CH_POST_NO_COOKIE;
 
-  return post_error ? OCHUSHA_UTILS_2CH_POST_FAILURE : OCHUSHA_UTILS_2CH_POST_SUCCESS;
+  return post_result
+    ? OCHUSHA_UTILS_2CH_POST_SUCCESS : OCHUSHA_UTILS_2CH_POST_FAILURE;
 }
