@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_utils_jbbs.c,v 1.10 2003/11/23 00:30:16 fuyu Exp $
+ * $Id: ochusha_utils_jbbs.c,v 1.11 2003/11/24 22:24:57 fuyu Exp $
  */
 
 #include "config.h"
@@ -240,7 +240,9 @@ convert_html_to_dat(WorkerThread *employee, HTML2DATConverterJobArgs *args)
 				"ochusha_utils_jbbs.c: convert_html_to_dat()"))
     {
       g_signal_emit_by_name(G_OBJECT(args->broker),
-			    "access_terminated", dat_buffer, &signal_result);
+			    "access_terminated",
+			    dat_buffer,
+			    &signal_result);
       goto terminated;
     }
 
@@ -248,7 +250,9 @@ convert_html_to_dat(WorkerThread *employee, HTML2DATConverterJobArgs *args)
 				"ochusha_utils_jbbs.c: convert_html_to_dat()"))
     {
       g_signal_emit_by_name(G_OBJECT(args->broker),
-			    "access_terminated", dat_buffer, &signal_result);
+			    "access_terminated",
+			    dat_buffer,
+			    &signal_result);
       ochusha_async_buffer_active_unref(html_buffer,
 				"ochusha_utils_jbbs.c: convert_html_to_dat()");
       goto terminated;
@@ -559,8 +563,10 @@ convert_html_to_dat(WorkerThread *employee, HTML2DATConverterJobArgs *args)
 #if DEBUG_HTML_TO_DAT_CONVERTER_MOST
 	  fprintf(stderr, "conversion done.\n");
 #endif
-	  g_signal_emit_by_name(G_OBJECT(args->broker), "access_completed",
-				dat_buffer, &signal_result);
+	  g_signal_emit_by_name(G_OBJECT(args->broker),
+				"access_completed",
+				dat_buffer,
+				&signal_result);
 
 	  if (previous_line != NULL)
 	    {
@@ -576,8 +582,10 @@ convert_html_to_dat(WorkerThread *employee, HTML2DATConverterJobArgs *args)
 #if DEBUG_HTML_TO_DAT_CONVERTER_MOST
 	  fprintf(stderr, "wait terminated.\n");
 #endif
-	  g_signal_emit_by_name(G_OBJECT(args->broker), "access_terminated",
-				dat_buffer, &signal_result);
+	  g_signal_emit_by_name(G_OBJECT(args->broker),
+				"access_terminated",
+				dat_buffer,
+				&signal_result);
 	  if (previous_line != NULL)
 	    {
 	      G_FREE(previous_line);
@@ -780,7 +788,11 @@ ochusha_utils_jbbs_fake_responses_source(OchushaThread2ch *thread_2ch,
 	{
 	  gboolean signal_result;
 	  g_signal_emit_by_name(G_OBJECT(broker),
-				"access_failed", dat_buffer, &signal_result);
+				"access_failed",
+				dat_buffer,
+				OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
+				_("Access failed: unknown reason"),
+				&signal_result);
 	  ochusha_async_buffer_fix(dat_buffer,
 				   "ochusha_utils_jbbs.c: ochusha_utils_jbbs_fake_responses_source()");
 	}
