@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: utils.c,v 1.12 2003/11/02 22:01:15 fuyu Exp $
+ * $Id: utils.c,v 1.13 2003/11/23 22:39:35 fuyu Exp $
  */
 
 #define OCHUSHA_UTILS_IMPLEMENTATION
@@ -1480,13 +1480,8 @@ cp932_to_utf8_helper(const char **inbuf, size_t *inbytesleft,
       return -1;
     }
 
-#if 1
   first = (*inbuf)[0];
   second = (*inbuf)[1];
-#else
-  first = (*inbuf)[0] & 0xff;
-  second = (*inbuf)[1] & 0xff;
-#endif
 
 #if DEBUG_ICONV
   fprintf(stderr, "first=%02x second=%02x\n", first, second);
